@@ -34,11 +34,54 @@ import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
  * using a {@link StaticTableOperations}. This way no Catalog related calls are needed when reading the table data after
  * deserialization.
  */
-abstract class BaseMetadataTable implements Table, Serializable {
+abstract class BaseMetadataTable implements Table, HasTableOperations, Serializable {
+  protected static final String PARTITION_FIELD_PREFIX = "partition.";
   private final PartitionSpec spec = PartitionSpec.unpartitioned();
   private final SortOrder sortOrder = SortOrder.unsorted();
+  private final TableOperations ops;
+  private final Table table;
+  private final String name;
 
-  abstract Table table();
+  protected BaseMetadataTable(TableOperations ops, Table table, String name) {
+    this.ops = ops;
+    this.table = table;
+    this.name = name;
+  }
+
+  /**
+   * This method transforms the table's partition spec to a spec that is used to rewrite the user-provided filter
+   * expression against the given metadata table.
+   * <p>
+   * The resulting partition spec maps $partitionPrefix.X fields to partition X using an identity partition transform.
+   * When this spec is used to project an expression for the given metadata table, the projection will remove
+   * predicates for non-partition fields (not in the spec) and will remove the "$partitionPrefix." prefix from fields.
+   *
+   * @param metadataTableSchena schema of the metadata table
+   * @param spec spec on which the metadata table schema is based
+   * @param partitionPrefix prefix to remove from each field in the partition spec
+   * @return a spec used to rewrite the metadata table filters to partition filters using an inclusive projection
+   */
+  static PartitionSpec transformSpec(Schema metadataTableSchena, PartitionSpec spec, String partitionPrefix) {
+    PartitionSpec.Builder identitySpecBuilder = PartitionSpec.builderFor(metadataTableSchena);
+    spec.fields().forEach(pf -> identitySpecBuilder.identity(partitionPrefix + pf.name(), pf.name()));
+    return identitySpecBuilder.build();
+  }
+
+  abstract MetadataTableType metadataTableType();
+
+  protected Table table() {
+    return table;
+  }
+
+  @Override
+  public TableOperations operations() {
+    return ops;
+  }
+
+  @Override
+  public String name() {
+    return name;
+  }
 
   @Override
   public FileIO io() {
@@ -65,6 +108,11 @@ abstract class BaseMetadataTable implements Table, Serializable {
     table().refresh();
   }
 
+  @Override
+  public Map<Integer, Schema> schemas() {
+    return ImmutableMap.of(TableMetadata.INITIAL_SCHEMA_ID, schema());
+  }
+
   @Override
   public PartitionSpec spec() {
     return spec;
@@ -195,39 +243,7 @@ abstract class BaseMetadataTable implements Table, Serializable {
     return name();
   }
 
-  abstract String metadataLocation();
-
-  abstract MetadataTableType metadataTableType();
-
   final Object writeReplace() {
-    return new TableProxy(io(), table().name(), name(), metadataLocation(), metadataTableType());
-  }
-
-  static class TableProxy implements Serializable {
-    private FileIO io;
-    private String baseTableName;
-    private String metadataTableName;
-    private String metadataLocation;
-    private MetadataTableType type;
-
-    TableProxy(FileIO io, String baseTableName, String metadataTableName, String metadataLocation,
-               MetadataTableType type) {
-      this.io = io;
-      this.baseTableName = baseTableName;
-      this.metadataTableName = metadataTableName;
-      this.metadataLocation = metadataLocation;
-      this.type = type;
-    }
-
-    /**
-     * Returns a table with {@link StaticTableOperations} so after deserialization no Catalog related calls are
-     * needed for accessing the table snapshot data.
-     * @return The metadata Table object for reading the table data at the time of the serialization of the original
-     *         object
-     */
-    private Object readResolve()  {
-      TableOperations ops = new StaticTableOperations(metadataLocation, io);
-      return MetadataTableUtils.createMetadataTableInstance(ops, baseTableName, metadataTableName, type);
-    }
+    return SerializableTable.copyOf(this);
   }
 }
