@@ -19,6 +19,7 @@
 
 package org.apache.iceberg;
 
+import java.io.Serializable;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -499,7 +500,12 @@ class BaseTransaction implements Transaction {
     }
   }
 
-  public class TransactionTable implements Table {
+  public class TransactionTable implements Table, HasTableOperations, Serializable {
+
+    @Override
+    public TableOperations operations() {
+      return transactionOps;
+    }
 
     @Override
     public String name() {
@@ -520,6 +526,11 @@ class BaseTransaction implements Transaction {
       return current.schema();
     }
 
+    @Override
+    public Map<Integer, Schema> schemas() {
+      return current.schemasById();
+    }
+
     @Override
     public PartitionSpec spec() {
       return current.spec();
@@ -674,6 +685,10 @@ class BaseTransaction implements Transaction {
     public String toString() {
       return name();
     }
+
+    Object writeReplace() {
+      return SerializableTable.copyOf(this);
+    }
   }
 
   @VisibleForTesting
