@@ -20,9 +20,11 @@
 package org.apache.iceberg;
 
 import java.io.Serializable;
-import java.util.Collections;
+import java.nio.ByteBuffer;
+import java.util.Arrays;
 import java.util.List;
 import java.util.function.Function;
+import java.util.stream.Stream;
 import org.apache.avro.Schema;
 import org.apache.avro.generic.IndexedRecord;
 import org.apache.avro.specific.SpecificData.SchemaConstructable;
@@ -32,6 +34,7 @@ import org.apache.iceberg.relocated.com.google.common.base.MoreObjects;
 import org.apache.iceberg.relocated.com.google.common.base.Objects;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.types.Types;
+import org.apache.iceberg.util.ByteBuffers;
 
 public class GenericManifestFile
     implements ManifestFile, StructLike, IndexedRecord, SchemaConstructable, Serializable {
@@ -56,7 +59,8 @@ public class GenericManifestFile
   private Long addedRowsCount = null;
   private Long existingRowsCount = null;
   private Long deletedRowsCount = null;
-  private List<PartitionFieldSummary> partitions = null;
+  private PartitionFieldSummary[] partitions = null;
+  private byte[] keyMetadata = null;
 
   /**
    * Used by Avro reflection to instantiate this class when reading manifest files.
@@ -100,13 +104,14 @@ public class GenericManifestFile
     this.deletedRowsCount = null;
     this.partitions = null;
     this.fromProjectionPos = null;
+    this.keyMetadata = null;
   }
 
   public GenericManifestFile(String path, long length, int specId, ManifestContent content,
                              long sequenceNumber, long minSequenceNumber, Long snapshotId,
                              int addedFilesCount, long addedRowsCount, int existingFilesCount,
                              long existingRowsCount, int deletedFilesCount, long deletedRowsCount,
-                             List<PartitionFieldSummary> partitions) {
+                             List<PartitionFieldSummary> partitions, ByteBuffer keyMetadata) {
     this.avroSchema = AVRO_SCHEMA;
     this.manifestPath = path;
     this.length = length;
@@ -121,8 +126,9 @@ public class GenericManifestFile
     this.existingRowsCount = existingRowsCount;
     this.deletedFilesCount = deletedFilesCount;
     this.deletedRowsCount = deletedRowsCount;
-    this.partitions = partitions;
+    this.partitions = partitions == null ? null : partitions.toArray(new PartitionFieldSummary[0]);
     this.fromProjectionPos = null;
+    this.keyMetadata = ByteBuffers.toByteArray(keyMetadata);
   }
 
   /**
@@ -145,8 +151,15 @@ public class GenericManifestFile
     this.existingRowsCount = toCopy.existingRowsCount;
     this.deletedFilesCount = toCopy.deletedFilesCount;
     this.deletedRowsCount = toCopy.deletedRowsCount;
-    this.partitions = copyList(toCopy.partitions, PartitionFieldSummary::copy);
+    if (toCopy.partitions != null) {
+      this.partitions = Stream.of(toCopy.partitions)
+          .map(PartitionFieldSummary::copy)
+          .toArray(PartitionFieldSummary[]::new);
+    } else {
+      this.partitions = null;
+    }
     this.fromProjectionPos = toCopy.fromProjectionPos;
+    this.keyMetadata = toCopy.keyMetadata == null ? null : Arrays.copyOf(toCopy.keyMetadata, toCopy.keyMetadata.length);
   }
 
   /**
@@ -235,7 +248,12 @@ public class GenericManifestFile
 
   @Override
   public List<PartitionFieldSummary> partitions() {
-    return partitions;
+    return partitions == null ? null : Arrays.asList(partitions);
+  }
+
+  @Override
+  public ByteBuffer keyMetadata() {
+    return keyMetadata == null ? null : ByteBuffer.wrap(keyMetadata);
   }
 
   @Override
@@ -283,7 +301,9 @@ public class GenericManifestFile
       case 12:
         return deletedRowsCount;
       case 13:
-        return partitions;
+        return partitions();
+      case 14:
+        return keyMetadata();
       default:
         throw new UnsupportedOperationException("Unknown field ordinal: " + pos);
     }
@@ -339,7 +359,11 @@ public class GenericManifestFile
         this.deletedRowsCount = (Long) value;
         return;
       case 13:
-        this.partitions = (List<PartitionFieldSummary>) value;
+        this.partitions = value == null ? null :
+            ((List<PartitionFieldSummary>) value).toArray(new PartitionFieldSummary[0]);
+        return;
+      case 14:
+        this.keyMetadata = ByteBuffers.toByteArray((ByteBuffer) value);
         return;
       default:
         // ignore the object, it must be from a newer version of the format
@@ -391,6 +415,7 @@ public class GenericManifestFile
         .add("deleted_data_files_count", deletedFilesCount)
         .add("deleted_rows_count", deletedRowsCount)
         .add("partitions", partitions)
+        .add("key_metadata", keyMetadata == null ? "null" : "(redacted)")
         .toString();
   }
 
@@ -410,7 +435,7 @@ public class GenericManifestFile
             toCopy.sequenceNumber(), toCopy.minSequenceNumber(), toCopy.snapshotId(),
             toCopy.addedFilesCount(), toCopy.addedRowsCount(), toCopy.existingFilesCount(),
             toCopy.existingRowsCount(), toCopy.deletedFilesCount(), toCopy.deletedRowsCount(),
-            copyList(toCopy.partitions(), PartitionFieldSummary::copy));
+            copyList(toCopy.partitions(), PartitionFieldSummary::copy), toCopy.keyMetadata());
       }
     }
 
@@ -430,7 +455,7 @@ public class GenericManifestFile
       for (E element : list) {
         copy.add(transform.apply(element));
       }
-      return Collections.unmodifiableList(copy);
+      return copy;
     }
     return null;
   }
