@@ -41,33 +41,17 @@ public class HistoryTable extends BaseMetadataTable {
       Types.NestedField.required(4, "is_current_ancestor", Types.BooleanType.get())
   );
 
-  private final TableOperations ops;
-  private final Table table;
-  private final String name;
-
   HistoryTable(TableOperations ops, Table table) {
     this(ops, table, table.name() + ".history");
   }
 
   HistoryTable(TableOperations ops, Table table, String name) {
-    this.ops = ops;
-    this.table = table;
-    this.name = name;
-  }
-
-  @Override
-  Table table() {
-    return table;
-  }
-
-  @Override
-  public String name() {
-    return name;
+    super(ops, table, name);
   }
 
   @Override
   public TableScan newScan() {
-    return new HistoryScan();
+    return new HistoryScan(operations(), table());
   }
 
   @Override
@@ -75,26 +59,23 @@ public class HistoryTable extends BaseMetadataTable {
     return HISTORY_SCHEMA;
   }
 
-  @Override
-  String metadataLocation() {
-    return ops.current().metadataFileLocation();
-  }
-
   @Override
   MetadataTableType metadataTableType() {
     return MetadataTableType.HISTORY;
   }
 
   private DataTask task(TableScan scan) {
+    TableOperations ops = operations();
     return StaticDataTask.of(
         ops.io().newInputFile(ops.current().metadataFileLocation()),
-        ops.current().snapshotLog(),
-        convertHistoryEntryFunc(table));
+        schema(), scan.schema(), ops.current().snapshotLog(),
+        convertHistoryEntryFunc(table())
+    );
   }
 
   private class HistoryScan extends StaticTableScan {
-    HistoryScan() {
-      super(ops, table, HISTORY_SCHEMA, HistoryTable.this::task);
+    HistoryScan(TableOperations ops, Table table) {
+      super(ops, table, HISTORY_SCHEMA, HistoryTable.this.metadataTableType().name(), HistoryTable.this::task);
     }
 
     @Override
