@@ -29,6 +29,7 @@ import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Predicate;
+import java.util.stream.Collectors;
 import org.apache.iceberg.exceptions.ValidationException;
 import org.apache.iceberg.io.InputFile;
 import org.apache.iceberg.relocated.com.google.common.base.MoreObjects;
@@ -40,7 +41,9 @@ import org.apache.iceberg.relocated.com.google.common.collect.Iterables;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.relocated.com.google.common.collect.Maps;
 import org.apache.iceberg.relocated.com.google.common.collect.Sets;
+import org.apache.iceberg.transforms.Transforms;
 import org.apache.iceberg.types.TypeUtil;
+import org.apache.iceberg.util.Pair;
 import org.apache.iceberg.util.PropertyUtil;
 
 /**
@@ -48,38 +51,39 @@ import org.apache.iceberg.util.PropertyUtil;
  */
 public class TableMetadata implements Serializable {
   static final long INITIAL_SEQUENCE_NUMBER = 0;
+  static final long INVALID_SEQUENCE_NUMBER = -1;
   static final int DEFAULT_TABLE_FORMAT_VERSION = 1;
   static final int SUPPORTED_TABLE_FORMAT_VERSION = 2;
   static final int INITIAL_SPEC_ID = 0;
   static final int INITIAL_SORT_ORDER_ID = 1;
+  static final int INITIAL_SCHEMA_ID = 0;
 
   private static final long ONE_MINUTE = TimeUnit.MINUTES.toMillis(1);
 
-  /**
-   * @deprecated will be removed in 0.9.0; use newTableMetadata(Schema, PartitionSpec, String, Map) instead.
-   */
-  @Deprecated
-  public static TableMetadata newTableMetadata(TableOperations ops,
-                                               Schema schema,
-                                               PartitionSpec spec,
-                                               String location,
-                                               Map<String, String> properties) {
-    return newTableMetadata(schema, spec, SortOrder.unsorted(), location, properties, DEFAULT_TABLE_FORMAT_VERSION);
-  }
-
   public static TableMetadata newTableMetadata(Schema schema,
                                                PartitionSpec spec,
                                                SortOrder sortOrder,
                                                String location,
                                                Map<String, String> properties) {
-    return newTableMetadata(schema, spec, sortOrder, location, properties, DEFAULT_TABLE_FORMAT_VERSION);
+    int formatVersion = PropertyUtil.propertyAsInt(properties, TableProperties.FORMAT_VERSION,
+        DEFAULT_TABLE_FORMAT_VERSION);
+    return newTableMetadata(schema, spec, sortOrder, location, unreservedProperties(properties), formatVersion);
   }
 
   public static TableMetadata newTableMetadata(Schema schema,
                                                PartitionSpec spec,
                                                String location,
                                                Map<String, String> properties) {
-    return newTableMetadata(schema, spec, SortOrder.unsorted(), location, properties, DEFAULT_TABLE_FORMAT_VERSION);
+    SortOrder sortOrder = SortOrder.unsorted();
+    int formatVersion = PropertyUtil.propertyAsInt(properties, TableProperties.FORMAT_VERSION,
+        DEFAULT_TABLE_FORMAT_VERSION);
+    return newTableMetadata(schema, spec, sortOrder, location, unreservedProperties(properties), formatVersion);
+  }
+
+  private static Map<String, String> unreservedProperties(Map<String, String> rawProperties) {
+    return rawProperties.entrySet().stream()
+        .filter(e -> !TableProperties.RESERVED_PROPERTIES.contains(e.getKey()))
+        .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
   }
 
   static TableMetadata newTableMetadata(Schema schema,
@@ -88,9 +92,12 @@ public class TableMetadata implements Serializable {
                                         String location,
                                         Map<String, String> properties,
                                         int formatVersion) {
+    Preconditions.checkArgument(properties.keySet().stream().noneMatch(TableProperties.RESERVED_PROPERTIES::contains),
+        "Table properties should not contain reserved properties, but got %s", properties);
+
     // reassign all column ids to ensure consistency
     AtomicInteger lastColumnId = new AtomicInteger(0);
-    Schema freshSchema = TypeUtil.assignFreshIds(schema, lastColumnId::incrementAndGet);
+    Schema freshSchema = TypeUtil.assignFreshIds(INITIAL_SCHEMA_ID, schema, lastColumnId::incrementAndGet);
 
     // rebuild the partition spec using the new column ids
     PartitionSpec.Builder specBuilder = PartitionSpec.builderFor(freshSchema)
@@ -116,7 +123,8 @@ public class TableMetadata implements Serializable {
 
     return new TableMetadata(null, formatVersion, UUID.randomUUID().toString(), location,
         INITIAL_SEQUENCE_NUMBER, System.currentTimeMillis(),
-        lastColumnId.get(), freshSchema, INITIAL_SPEC_ID, ImmutableList.of(freshSpec),
+        lastColumnId.get(), freshSchema.schemaId(), ImmutableList.of(freshSchema),
+        freshSpec.specId(), ImmutableList.of(freshSpec), freshSpec.lastAssignedFieldId(),
         freshSortOrderId, ImmutableList.of(freshSortOrder),
         ImmutableMap.copyOf(properties), -1, ImmutableList.of(),
         ImmutableList.of(), ImmutableList.of());
@@ -219,15 +227,18 @@ public class TableMetadata implements Serializable {
   private final long lastSequenceNumber;
   private final long lastUpdatedMillis;
   private final int lastColumnId;
-  private final Schema schema;
+  private final int currentSchemaId;
+  private final List<Schema> schemas;
   private final int defaultSpecId;
   private final List<PartitionSpec> specs;
+  private final int lastAssignedPartitionId;
   private final int defaultSortOrderId;
   private final List<SortOrder> sortOrders;
   private final Map<String, String> properties;
   private final long currentSnapshotId;
   private final List<Snapshot> snapshots;
   private final Map<Long, Snapshot> snapshotsById;
+  private final Map<Integer, Schema> schemasById;
   private final Map<Integer, PartitionSpec> specsById;
   private final Map<Integer, SortOrder> sortOrdersById;
   private final List<HistoryEntry> snapshotLog;
@@ -241,9 +252,11 @@ public class TableMetadata implements Serializable {
                 long lastSequenceNumber,
                 long lastUpdatedMillis,
                 int lastColumnId,
-                Schema schema,
+                int currentSchemaId,
+                List<Schema> schemas,
                 int defaultSpecId,
                 List<PartitionSpec> specs,
+                int lastAssignedPartitionId,
                 int defaultSortOrderId,
                 List<SortOrder> sortOrders,
                 Map<String, String> properties,
@@ -268,9 +281,11 @@ public class TableMetadata implements Serializable {
     this.lastSequenceNumber = lastSequenceNumber;
     this.lastUpdatedMillis = lastUpdatedMillis;
     this.lastColumnId = lastColumnId;
-    this.schema = schema;
+    this.currentSchemaId = currentSchemaId;
+    this.schemas = schemas;
     this.specs = specs;
     this.defaultSpecId = defaultSpecId;
+    this.lastAssignedPartitionId = lastAssignedPartitionId;
     this.defaultSortOrderId = defaultSortOrderId;
     this.sortOrders = sortOrders;
     this.properties = properties;
@@ -280,6 +295,7 @@ public class TableMetadata implements Serializable {
     this.previousFiles = previousFiles;
 
     this.snapshotsById = indexAndValidateSnapshots(snapshots, lastSequenceNumber);
+    this.schemasById = indexSchemas();
     this.specsById = indexSpecs(specs);
     this.sortOrdersById = indexSortOrders(sortOrders);
 
@@ -356,7 +372,19 @@ public class TableMetadata implements Serializable {
   }
 
   public Schema schema() {
-    return schema;
+    return schemasById.get(currentSchemaId);
+  }
+
+  public List<Schema> schemas() {
+    return schemas;
+  }
+
+  public Map<Integer, Schema> schemasById() {
+    return schemasById;
+  }
+
+  public int currentSchemaId() {
+    return currentSchemaId;
   }
 
   public PartitionSpec spec() {
@@ -375,6 +403,10 @@ public class TableMetadata implements Serializable {
     return specsById;
   }
 
+  int lastAssignedPartitionId() {
+    return lastAssignedPartitionId;
+  }
+
   public int defaultSpecId() {
     return defaultSpecId;
   }
@@ -444,9 +476,9 @@ public class TableMetadata implements Serializable {
       return this;
     } else {
       return new TableMetadata(null, formatVersion, UUID.randomUUID().toString(), location,
-          lastSequenceNumber, lastUpdatedMillis, lastColumnId, schema, defaultSpecId, specs,
-          defaultSortOrderId, sortOrders, properties, currentSnapshotId, snapshots, snapshotLog,
-          addPreviousFile(file, lastUpdatedMillis));
+          lastSequenceNumber, lastUpdatedMillis, lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+          lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties,
+          currentSnapshotId, snapshots, snapshotLog, addPreviousFile(file, lastUpdatedMillis));
     }
   }
 
@@ -456,14 +488,29 @@ public class TableMetadata implements Serializable {
     // rebuild all of the partition specs and sort orders for the new current schema
     List<PartitionSpec> updatedSpecs = Lists.transform(specs, spec -> updateSpecSchema(newSchema, spec));
     List<SortOrder> updatedSortOrders = Lists.transform(sortOrders, order -> updateSortOrderSchema(newSchema, order));
+
+    int newSchemaId = reuseOrCreateNewSchemaId(newSchema);
+    if (currentSchemaId == newSchemaId && newLastColumnId == lastColumnId) {
+      // the new spec and last column Id is already current and no change is needed
+      return this;
+    }
+
+    ImmutableList.Builder<Schema> builder = ImmutableList.<Schema>builder().addAll(schemas);
+    if (!schemasById.containsKey(newSchemaId)) {
+      builder.add(new Schema(newSchemaId, newSchema.columns(), newSchema.identifierFieldIds()));
+    }
+
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), newLastColumnId, newSchema, defaultSpecId, updatedSpecs,
+        lastSequenceNumber, System.currentTimeMillis(), newLastColumnId,
+        newSchemaId, builder.build(), defaultSpecId, updatedSpecs, lastAssignedPartitionId,
         defaultSortOrderId, updatedSortOrders, properties, currentSnapshotId, snapshots, snapshotLog,
         addPreviousFile(file, lastUpdatedMillis));
   }
 
   // The caller is responsible to pass a newPartitionSpec with correct partition field IDs
   public TableMetadata updatePartitionSpec(PartitionSpec newPartitionSpec) {
+    Schema schema = schema();
+
     PartitionSpec.checkCompatibility(newPartitionSpec, schema);
     ValidationException.check(formatVersion > 1 || PartitionSpec.hasSequentialIds(newPartitionSpec),
         "Spec does not use sequential IDs that are required in v1: %s", newPartitionSpec);
@@ -492,12 +539,14 @@ public class TableMetadata implements Serializable {
     }
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, newDefaultSpecId,
-        builder.build(), defaultSortOrderId, sortOrders, properties,
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, newDefaultSpecId,
+        builder.build(), Math.max(lastAssignedPartitionId, newPartitionSpec.lastAssignedFieldId()),
+        defaultSortOrderId, sortOrders, properties,
         currentSnapshotId, snapshots, snapshotLog, addPreviousFile(file, lastUpdatedMillis));
   }
 
   public TableMetadata replaceSortOrder(SortOrder newOrder) {
+    Schema schema = schema();
     SortOrder.checkCompatibility(newOrder, schema);
 
     // determine the next order id
@@ -529,8 +578,8 @@ public class TableMetadata implements Serializable {
     }
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        newOrderId, builder.build(), properties, currentSnapshotId, snapshots, snapshotLog,
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, newOrderId, builder.build(), properties, currentSnapshotId, snapshots, snapshotLog,
         addPreviousFile(file, lastUpdatedMillis));
   }
 
@@ -545,7 +594,8 @@ public class TableMetadata implements Serializable {
         .build();
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        snapshot.sequenceNumber(), snapshot.timestampMillis(), lastColumnId, schema, defaultSpecId, specs,
+        snapshot.sequenceNumber(), snapshot.timestampMillis(), lastColumnId,
+        currentSchemaId, schemas, defaultSpecId, specs, lastAssignedPartitionId,
         defaultSortOrderId, sortOrders, properties, currentSnapshotId, newSnapshots, snapshotLog,
         addPreviousFile(file, lastUpdatedMillis));
   }
@@ -570,7 +620,8 @@ public class TableMetadata implements Serializable {
         .build();
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        snapshot.sequenceNumber(), snapshot.timestampMillis(), lastColumnId, schema, defaultSpecId, specs,
+        snapshot.sequenceNumber(), snapshot.timestampMillis(), lastColumnId,
+        currentSchemaId, schemas, defaultSpecId, specs, lastAssignedPartitionId,
         defaultSortOrderId, sortOrders, properties, snapshot.snapshotId(), newSnapshots, newSnapshotLog,
         addPreviousFile(file, lastUpdatedMillis));
   }
@@ -602,8 +653,8 @@ public class TableMetadata implements Serializable {
     }
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        defaultSortOrderId, sortOrders, properties, currentSnapshotId, filtered,
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties, currentSnapshotId, filtered,
         ImmutableList.copyOf(newSnapshotLog), addPreviousFile(file, lastUpdatedMillis));
   }
 
@@ -626,17 +677,25 @@ public class TableMetadata implements Serializable {
         .build();
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, nowMillis, lastColumnId, schema, defaultSpecId, specs, defaultSortOrderId,
-        sortOrders, properties, snapshot.snapshotId(), snapshots, newSnapshotLog,
-        addPreviousFile(file, lastUpdatedMillis));
+        lastSequenceNumber, nowMillis, lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties, snapshot.snapshotId(), snapshots,
+        newSnapshotLog, addPreviousFile(file, lastUpdatedMillis));
   }
 
-  public TableMetadata replaceProperties(Map<String, String> newProperties) {
-    ValidationException.check(newProperties != null, "Cannot set properties to null");
-    return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        defaultSortOrderId, sortOrders, newProperties, currentSnapshotId, snapshots, snapshotLog,
-        addPreviousFile(file, lastUpdatedMillis, newProperties));
+  public TableMetadata replaceProperties(Map<String, String> rawProperties) {
+    ValidationException.check(rawProperties != null, "Cannot set properties to null");
+    Map<String, String> newProperties = unreservedProperties(rawProperties);
+    TableMetadata metadata = new TableMetadata(null, formatVersion, uuid, location,
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, newProperties, currentSnapshotId, snapshots,
+        snapshotLog, addPreviousFile(file, lastUpdatedMillis, newProperties));
+
+    int newFormatVersion = PropertyUtil.propertyAsInt(rawProperties, TableProperties.FORMAT_VERSION, formatVersion);
+    if (formatVersion != newFormatVersion) {
+      metadata = metadata.upgradeToFormatVersion(newFormatVersion);
+    }
+
+    return metadata;
   }
 
   public TableMetadata removeSnapshotLogEntries(Set<Long> snapshotIds) {
@@ -653,9 +712,60 @@ public class TableMetadata implements Serializable {
         "Cannot set invalid snapshot log: latest entry is not the current snapshot");
 
     return new TableMetadata(null, formatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        defaultSortOrderId, sortOrders, properties, currentSnapshotId, snapshots, newSnapshotLog,
-        addPreviousFile(file, lastUpdatedMillis));
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties, currentSnapshotId,
+        snapshots, newSnapshotLog, addPreviousFile(file, lastUpdatedMillis));
+  }
+
+  private PartitionSpec reassignPartitionIds(PartitionSpec partitionSpec, TypeUtil.NextID nextID) {
+    PartitionSpec.Builder specBuilder = PartitionSpec.builderFor(partitionSpec.schema())
+        .withSpecId(partitionSpec.specId());
+
+    if (formatVersion > 1) {
+      // for v2 and later, reuse any existing field IDs, but reproduce the same spec
+      Map<Pair<Integer, String>, Integer> transformToFieldId = specs.stream()
+          .flatMap(spec -> spec.fields().stream())
+          .collect(Collectors.toMap(
+              field -> Pair.of(field.sourceId(), field.transform().toString()),
+              PartitionField::fieldId,
+              Math::max));
+
+      for (PartitionField field : partitionSpec.fields()) {
+        // reassign the partition field ids
+        int partitionFieldId = transformToFieldId.computeIfAbsent(
+            Pair.of(field.sourceId(), field.transform().toString()), k -> nextID.get());
+        specBuilder.add(
+            field.sourceId(),
+            partitionFieldId,
+            field.name(),
+            field.transform());
+      }
+
+    } else {
+      // for v1, preserve the existing spec and carry forward all fields, replacing missing fields with void
+      Map<Pair<Integer, String>, PartitionField> newFields = Maps.newLinkedHashMap();
+      for (PartitionField newField : partitionSpec.fields()) {
+        newFields.put(Pair.of(newField.sourceId(), newField.transform().toString()), newField);
+      }
+
+      for (PartitionField field : spec().fields()) {
+        // ensure each field is either carried forward or replaced with void
+        PartitionField newField = newFields.remove(Pair.of(field.sourceId(), field.transform().toString()));
+        if (newField != null) {
+          // copy the new field with the existing field ID
+          specBuilder.add(newField.sourceId(), field.fieldId(), newField.name(), newField.transform());
+        } else {
+          specBuilder.add(field.sourceId(), field.fieldId(), field.name(), Transforms.alwaysNull());
+        }
+      }
+
+      // add any remaining new fields at the end and assign new partition field IDs
+      for (PartitionField newField : newFields.values()) {
+        specBuilder.add(newField.sourceId(), nextID.get(), newField.name(), newField.transform());
+      }
+    }
+
+    return specBuilder.build();
   }
 
   // The caller is responsible to pass a updatedPartitionSpec with correct partition field IDs
@@ -666,7 +776,7 @@ public class TableMetadata implements Serializable {
         "Spec does not use sequential IDs that are required in v1: %s", updatedPartitionSpec);
 
     AtomicInteger newLastColumnId = new AtomicInteger(lastColumnId);
-    Schema freshSchema = TypeUtil.assignFreshIds(updatedSchema, schema, newLastColumnId::incrementAndGet);
+    Schema freshSchema = TypeUtil.assignFreshIds(updatedSchema, schema(), newLastColumnId::incrementAndGet);
 
     // determine the next spec id
     OptionalInt maxSpecId = specs.stream().mapToInt(PartitionSpec::specId).max();
@@ -675,16 +785,20 @@ public class TableMetadata implements Serializable {
     // rebuild the partition spec using the new column ids
     PartitionSpec freshSpec = freshSpec(nextSpecId, freshSchema, updatedPartitionSpec);
 
+    // reassign partition field ids with existing partition specs in the table
+    AtomicInteger lastPartitionId = new AtomicInteger(lastAssignedPartitionId);
+    PartitionSpec newSpec = reassignPartitionIds(freshSpec, lastPartitionId::incrementAndGet);
+
     // if the spec already exists, use the same ID. otherwise, use 1 more than the highest ID.
     int specId = specs.stream()
-        .filter(freshSpec::compatibleWith)
+        .filter(newSpec::compatibleWith)
         .findFirst()
         .map(PartitionSpec::specId)
         .orElse(nextSpecId);
 
     ImmutableList.Builder<PartitionSpec> specListBuilder = ImmutableList.<PartitionSpec>builder().addAll(specs);
     if (!specsById.containsKey(specId)) {
-      specListBuilder.add(freshSpec);
+      specListBuilder.add(newSpec);
     }
 
     // determine the next order id
@@ -708,19 +822,37 @@ public class TableMetadata implements Serializable {
 
     Map<String, String> newProperties = Maps.newHashMap();
     newProperties.putAll(this.properties);
-    newProperties.putAll(updatedProperties);
+    newProperties.putAll(unreservedProperties(updatedProperties));
 
-    return new TableMetadata(null, formatVersion, uuid, newLocation,
-        lastSequenceNumber, System.currentTimeMillis(), newLastColumnId.get(), freshSchema,
-        specId, specListBuilder.build(), orderId, sortOrdersBuilder.build(), ImmutableMap.copyOf(newProperties),
+    // check if there is format version override
+    int newFormatVersion = PropertyUtil.propertyAsInt(updatedProperties, TableProperties.FORMAT_VERSION, formatVersion);
+
+    // determine the next schema id
+    int freshSchemaId = reuseOrCreateNewSchemaId(freshSchema);
+    ImmutableList.Builder<Schema> schemasBuilder = ImmutableList.<Schema>builder().addAll(schemas);
+
+    if (!schemasById.containsKey(freshSchemaId)) {
+      schemasBuilder.add(new Schema(freshSchemaId, freshSchema.columns(), freshSchema.identifierFieldIds()));
+    }
+
+    TableMetadata metadata = new TableMetadata(null, formatVersion, uuid, newLocation,
+        lastSequenceNumber, System.currentTimeMillis(), newLastColumnId.get(), freshSchemaId, schemasBuilder.build(),
+        specId, specListBuilder.build(), Math.max(lastAssignedPartitionId, newSpec.lastAssignedFieldId()),
+        orderId, sortOrdersBuilder.build(), ImmutableMap.copyOf(newProperties),
         -1, snapshots, ImmutableList.of(), addPreviousFile(file, lastUpdatedMillis, newProperties));
+
+    if (formatVersion != newFormatVersion) {
+      metadata = metadata.upgradeToFormatVersion(newFormatVersion);
+    }
+
+    return metadata;
   }
 
   public TableMetadata updateLocation(String newLocation) {
     return new TableMetadata(null, formatVersion, uuid, newLocation,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        defaultSortOrderId, sortOrders, properties, currentSnapshotId, snapshots, snapshotLog,
-        addPreviousFile(file, lastUpdatedMillis));
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties, currentSnapshotId,
+        snapshots, snapshotLog, addPreviousFile(file, lastUpdatedMillis));
   }
 
   public TableMetadata upgradeToFormatVersion(int newFormatVersion) {
@@ -735,9 +867,9 @@ public class TableMetadata implements Serializable {
     }
 
     return new TableMetadata(null, newFormatVersion, uuid, location,
-        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, schema, defaultSpecId, specs,
-        defaultSortOrderId, sortOrders, properties, currentSnapshotId, snapshots, snapshotLog,
-        addPreviousFile(file, lastUpdatedMillis));
+        lastSequenceNumber, System.currentTimeMillis(), lastColumnId, currentSchemaId, schemas, defaultSpecId, specs,
+        lastAssignedPartitionId, defaultSortOrderId, sortOrders, properties, currentSnapshotId,
+        snapshots, snapshotLog, addPreviousFile(file, lastUpdatedMillis));
   }
 
   private List<MetadataLogEntry> addPreviousFile(InputFile previousFile, long timestampMillis) {
@@ -771,7 +903,7 @@ public class TableMetadata implements Serializable {
 
     // add all of the fields to the builder. IDs should not change.
     for (PartitionField field : partitionSpec.fields()) {
-      specBuilder.add(field.sourceId(), field.fieldId(), field.name(), field.transform().toString());
+      specBuilder.add(field.sourceId(), field.fieldId(), field.name(), field.transform());
     }
 
     return specBuilder.build();
@@ -834,6 +966,14 @@ public class TableMetadata implements Serializable {
     return builder.build();
   }
 
+  private Map<Integer, Schema> indexSchemas() {
+    ImmutableMap.Builder<Integer, Schema> builder = ImmutableMap.builder();
+    for (Schema schema : schemas) {
+      builder.put(schema.schemaId(), schema);
+    }
+    return builder.build();
+  }
+
   private static Map<Integer, PartitionSpec> indexSpecs(List<PartitionSpec> specs) {
     ImmutableMap.Builder<Integer, PartitionSpec> builder = ImmutableMap.builder();
     for (PartitionSpec spec : specs) {
@@ -849,4 +989,18 @@ public class TableMetadata implements Serializable {
     }
     return builder.build();
   }
+
+  private int reuseOrCreateNewSchemaId(Schema newSchema) {
+    // if the schema already exists, use its id; otherwise use the highest id + 1
+    int newSchemaId = currentSchemaId;
+    for (Schema schema : schemas) {
+      if (schema.sameSchema(newSchema)) {
+        newSchemaId = schema.schemaId();
+        break;
+      } else if (schema.schemaId() >= newSchemaId) {
+        newSchemaId = schema.schemaId() + 1;
+      }
+    }
+    return newSchemaId;
+  }
 }
