@@ -582,8 +582,7 @@ public class ValueReaders {
 
     protected StructReader(List<ValueReader<?>> readers, Schema schema) {
       this.readers = readers.toArray(new ValueReader[0]);
-      this.positions = new int[0];
-      this.constants = new Object[0];
+      Integer isDeletedColumnPos = null;
 
       List<Schema.Field> fields = schema.getFields();
       for (int pos = 0; pos < fields.size(); pos += 1) {
@@ -591,8 +590,18 @@ public class ValueReaders {
         if (AvroSchemaUtil.getFieldId(field) == MetadataColumns.ROW_POSITION.fieldId()) {
           // track where the _pos field is located for setRowPositionSupplier
           this.posField = pos;
+        } else if (AvroSchemaUtil.getFieldId(field) == MetadataColumns.IS_DELETED.fieldId()) {
+          isDeletedColumnPos = pos;
         }
       }
+
+      if (isDeletedColumnPos == null) {
+        this.positions = new int[0];
+        this.constants = new Object[0];
+      } else {
+        this.positions = new int[]{isDeletedColumnPos};
+        this.constants = new Object[]{false};
+      }
     }
 
     protected StructReader(List<ValueReader<?>> readers, Types.StructType struct, Map<Integer, ?> idToConstant) {
@@ -609,6 +618,9 @@ public class ValueReaders {
         } else if (field.fieldId() == MetadataColumns.ROW_POSITION.fieldId()) {
           // track where the _pos field is located for setRowPositionSupplier
           this.posField = pos;
+        } else if (field.fieldId() == MetadataColumns.IS_DELETED.fieldId()) {
+          positionList.add(pos);
+          constantList.add(false);
         }
       }
 
