@@ -35,6 +35,7 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.function.Predicate;
+import java.util.stream.Stream;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -568,6 +569,10 @@ public class Tasks {
     return new Builder<>(Arrays.asList(items));
   }
 
+  public static <I> Builder<I> foreach(Stream<I> items) {
+    return new Builder<>(() -> items.iterator());
+  }
+
   @SuppressWarnings("unchecked")
   private static <E extends Exception> void throwOne(
       Collection<Throwable> exceptions, Class<E> allowedException) throws E {
