@@ -29,6 +29,7 @@ import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableSet;
 import org.apache.iceberg.types.Types;
 import org.junit.Assert;
+import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -71,6 +72,14 @@ public class TestOverwriteWithValidation extends TableTestBase {
       ))
       .build();
 
+  private static final DeleteFile FILE_DAY_1_POS_DELETES = FileMetadata.deleteFileBuilder(PARTITION_SPEC)
+      .ofPositionDeletes()
+      .withPath("/path/to/data-1-deletes.parquet")
+      .withFileSizeInBytes(10)
+      .withPartitionPath("date=2018-06-08")
+      .withRecordCount(1)
+      .build();
+
   private static final DataFile FILE_DAY_2 = DataFiles
       .builder(PARTITION_SPEC)
       .withPath("/path/to/data-2.parquet")
@@ -85,6 +94,22 @@ public class TestOverwriteWithValidation extends TableTestBase {
       ))
       .build();
 
+  private static final DeleteFile FILE_DAY_2_EQ_DELETES = FileMetadata.deleteFileBuilder(PARTITION_SPEC)
+      .ofEqualityDeletes()
+      .withPath("/path/to/data-2-eq-deletes.parquet")
+      .withFileSizeInBytes(10)
+      .withPartitionPath("date=2018-06-09")
+      .withRecordCount(1)
+      .build();
+
+  private static final DeleteFile FILE_DAY_2_POS_DELETES = FileMetadata.deleteFileBuilder(PARTITION_SPEC)
+      .ofPositionDeletes()
+      .withPath("/path/to/data-2-deletes.parquet")
+      .withFileSizeInBytes(10)
+      .withPartitionPath("date=2018-06-09")
+      .withRecordCount(1)
+      .build();
+
   private static final DataFile FILE_DAY_2_MODIFIED = DataFiles
       .builder(PARTITION_SPEC)
       .withPath("/path/to/data-3.parquet")
@@ -113,6 +138,21 @@ public class TestOverwriteWithValidation extends TableTestBase {
       ))
       .build();
 
+  private static final DeleteFile FILE_DAY_2_ANOTHER_RANGE_EQ_DELETES = FileMetadata.deleteFileBuilder(PARTITION_SPEC)
+      .ofEqualityDeletes()
+      .withPath("/path/to/data-3-eq-deletes.parquet")
+      .withFileSizeInBytes(10)
+      .withPartitionPath("date=2018-06-09")
+      .withRecordCount(1)
+      .withMetrics(new Metrics(1L,
+          null, // no column sizes
+          ImmutableMap.of(1, 1L, 2, 1L), // value count
+          ImmutableMap.of(1, 0L, 2, 0L), // null count
+          ImmutableMap.of(1, longToBuffer(10L)), // lower bounds
+          ImmutableMap.of(1, longToBuffer(10L)) // upper bounds
+      ))
+      .build();
+
   private static final Expression EXPRESSION_DAY_2 = equal("date", "2018-06-09");
 
   private static final Expression EXPRESSION_DAY_2_ID_RANGE = and(
@@ -611,4 +651,218 @@ public class TestOverwriteWithValidation extends TableTestBase {
     Assert.assertEquals("Should not create a new snapshot",
         committedSnapshotId, table.currentSnapshot().snapshotId());
   }
+
+  @Test
+  public void testConcurrentConflictingPositionDeletes() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .deleteFile(FILE_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_2_POS_DELETES)
+        .commit();
+
+    AssertHelpers.assertThrows("Should reject commit",
+        ValidationException.class, "found new delete",
+        overwrite::commit);
+  }
+
+  @Test
+  public void testConcurrentConflictingPositionDeletesOverwriteByFilter() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .overwriteByRowFilter(EXPRESSION_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_2_POS_DELETES)
+        .commit();
+
+    AssertHelpers.assertThrows("Should reject commit",
+        ValidationException.class, "Found new conflicting delete",
+        overwrite::commit);
+  }
+
+  @Test
+  public void testConcurrentNonConflictingPositionDeletes() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .deleteFile(FILE_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_1_POS_DELETES)
+        .commit();
+
+    overwrite.commit();
+
+    validateTableFiles(table, FILE_DAY_1, FILE_DAY_2_MODIFIED);
+    validateTableDeleteFiles(table, FILE_DAY_1_POS_DELETES);
+  }
+
+  @Test
+  public void testConcurrentNonConflictingPositionDeletesOverwriteByFilter() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .overwriteByRowFilter(EXPRESSION_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_1_POS_DELETES)
+        .commit();
+
+    overwrite.commit();
+
+    validateTableFiles(table, FILE_DAY_1, FILE_DAY_2_MODIFIED);
+    validateTableDeleteFiles(table, FILE_DAY_1_POS_DELETES);
+  }
+
+  @Test
+  public void testConcurrentConflictingEqualityDeletes() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .deleteFile(FILE_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_2_EQ_DELETES)
+        .commit();
+
+    AssertHelpers.assertThrows("Should reject commit",
+        ValidationException.class, "found new delete",
+        overwrite::commit);
+  }
+
+  @Test
+  public void testConcurrentNonConflictingEqualityDeletes() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_2)
+        .appendFile(FILE_DAY_2_ANOTHER_RANGE)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .deleteFile(FILE_DAY_2)
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .conflictDetectionFilter(EXPRESSION_DAY_2_ID_RANGE)
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_2_ANOTHER_RANGE_EQ_DELETES)
+        .commit();
+
+    overwrite.commit();
+
+    validateTableFiles(table, FILE_DAY_2_ANOTHER_RANGE, FILE_DAY_2_MODIFIED);
+    validateTableDeleteFiles(table, FILE_DAY_2_ANOTHER_RANGE_EQ_DELETES);
+  }
+
+  @Test
+  public void testOverwriteByFilterInheritsConflictDetectionFilter() {
+    Assume.assumeTrue(formatVersion == 2);
+
+    Assert.assertNull("Should be empty table", table.currentSnapshot());
+
+    table.newAppend()
+        .appendFile(FILE_DAY_1)
+        .appendFile(FILE_DAY_2)
+        .commit();
+
+    Snapshot firstSnapshot = table.currentSnapshot();
+
+    OverwriteFiles overwrite = table.newOverwrite()
+        .overwriteByRowFilter(EXPRESSION_DAY_2)
+        .validateAddedFilesMatchOverwriteFilter()
+        .addFile(FILE_DAY_2_MODIFIED)
+        .validateFromSnapshot(firstSnapshot.snapshotId())
+        .validateNoConflictingData()
+        .validateNoConflictingDeletes();
+
+    table.newRowDelta()
+        .addDeletes(FILE_DAY_1_POS_DELETES)
+        .commit();
+
+    overwrite.commit();
+
+    validateTableFiles(table, FILE_DAY_1, FILE_DAY_2_MODIFIED);
+    validateTableDeleteFiles(table, FILE_DAY_1_POS_DELETES);
+  }
 }
