@@ -501,6 +501,61 @@ public class TestSchemaUpdate {
     );
   }
 
+  @Test
+  public void testDeleteThenAdd() {
+    Schema schema = new Schema(required(1, "id", Types.IntegerType.get()));
+    Schema expected = new Schema(optional(2, "id", Types.IntegerType.get()));
+
+    Schema updated = new SchemaUpdate(schema, 1)
+        .deleteColumn("id")
+        .addColumn("id", optional(2, "id", Types.IntegerType.get()).type())
+        .apply();
+
+    Assert.assertEquals("Should match with added fields", expected.asStruct(), updated.asStruct());
+  }
+
+  @Test
+  public void testDeleteThenAddNested() {
+    Schema expectedNested = new Schema(
+        required(1, "id", Types.IntegerType.get()),
+        optional(2, "data", Types.StringType.get()),
+        optional(3, "preferences", Types.StructType.of(
+            optional(9, "feature2", Types.BooleanType.get()),
+            optional(24, "feature1", Types.BooleanType.get())
+        ), "struct of named boolean options"),
+        required(4, "locations", Types.MapType.ofRequired(10, 11,
+            Types.StructType.of(
+                    required(20, "address", Types.StringType.get()),
+                    required(21, "city", Types.StringType.get()),
+                    required(22, "state", Types.StringType.get()),
+                    required(23, "zip", Types.IntegerType.get())
+            ),
+            Types.StructType.of(
+                    required(12, "lat", Types.FloatType.get()),
+                    required(13, "long", Types.FloatType.get())
+            )), "map of address to coordinate"),
+        optional(5, "points", Types.ListType.ofOptional(14,
+            Types.StructType.of(
+                    required(15, "x", Types.LongType.get()),
+                    required(16, "y", Types.LongType.get())
+            )), "2-D cartesian points"),
+        required(6, "doubles", Types.ListType.ofRequired(17,
+            Types.DoubleType.get()
+        )),
+        optional(7, "properties", Types.MapType.ofOptional(18, 19,
+            Types.StringType.get(),
+            Types.StringType.get()
+        ), "string map of properties")
+    );
+
+    Schema updatedNested = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .deleteColumn("preferences.feature1")
+        .addColumn("preferences", "feature1", Types.BooleanType.get())
+        .apply();
+
+    Assert.assertEquals("Should match with added fields", expectedNested.asStruct(), updatedNested.asStruct());
+  }
+
   @Test
   public void testDeleteMissingColumn() {
     AssertHelpers.assertThrows("Should reject delete missing column",
@@ -1168,4 +1223,300 @@ public class TestSchemaUpdate {
                 .moveBefore("s2.x", "s1.a")
                 .apply());
   }
+
+  @Test
+  public void testAddExistingIdentifierFields() {
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("id")
+        .apply();
+
+    Assert.assertEquals("add an existing field as identifier field should succeed",
+        Sets.newHashSet(newSchema.findField("id").fieldId()),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testAddNewIdentifierFieldColumns() {
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn("new_field", Types.StringType.get())
+        .setIdentifierFields("id", "new_field")
+        .apply();
+
+    Assert.assertEquals("add column then set as identifier should succeed",
+        Sets.newHashSet(newSchema.findField("id").fieldId(), newSchema.findField("new_field").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .setIdentifierFields("id", "new_field")
+        .addRequiredColumn("new_field", Types.StringType.get())
+        .apply();
+
+    Assert.assertEquals("set identifier then add column should succeed",
+        Sets.newHashSet(newSchema.findField("id").fieldId(), newSchema.findField("new_field").fieldId()),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testAddNestedIdentifierFieldColumns() {
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("preferences.feature1")
+        .apply();
+
+    Assert.assertEquals("set existing nested field as identifier should succeed",
+        Sets.newHashSet(newSchema.findField("preferences.feature1").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn("new", Types.StructType.of(
+            Types.NestedField.required(SCHEMA_LAST_COLUMN_ID + 1, "field", Types.StringType.get())
+        ))
+        .setIdentifierFields("new.field")
+        .apply();
+
+    Assert.assertEquals("set newly added nested field as identifier should succeed",
+        Sets.newHashSet(newSchema.findField("new.field").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn("new", Types.StructType.of(
+            Types.NestedField.required(SCHEMA_LAST_COLUMN_ID + 1, "field", Types.StructType.of(
+                Types.NestedField.required(SCHEMA_LAST_COLUMN_ID + 2, "nested", Types.StringType.get())))))
+        .setIdentifierFields("new.field.nested")
+        .apply();
+
+    Assert.assertEquals("set newly added multi-layer nested field as identifier should succeed",
+        Sets.newHashSet(newSchema.findField("new.field.nested").fieldId()),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testAddDottedIdentifierFieldColumns() {
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn(null, "dot.field", Types.StringType.get())
+        .setIdentifierFields("id", "dot.field")
+        .apply();
+
+    Assert.assertEquals("add a field with dot as identifier should succeed",
+        Sets.newHashSet(newSchema.findField("id").fieldId(), newSchema.findField("dot.field").fieldId()),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testRemoveIdentifierFields() {
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn("new_field", Types.StringType.get())
+        .addRequiredColumn("new_field2", Types.StringType.get())
+        .setIdentifierFields("id", "new_field", "new_field2")
+        .apply();
+
+    newSchema = new SchemaUpdate(newSchema, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("new_field", "new_field2")
+        .apply();
+
+    Assert.assertEquals("remove an identifier field should succeed",
+        Sets.newHashSet(newSchema.findField("new_field").fieldId(), newSchema.findField("new_field2").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(newSchema, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields(Sets.newHashSet())
+        .apply();
+
+    Assert.assertEquals("remove all identifier fields should succeed",
+        Sets.newHashSet(),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testSetIdentifierFieldsFails() {
+
+    Schema testSchema = new Schema(
+        optional(1, "id", Types.IntegerType.get()),
+        required(2, "float", Types.FloatType.get()),
+        required(3, "double", Types.DoubleType.get())
+    );
+
+    AssertHelpers.assertThrows("Creating schema with optional identifier field should fail",
+        IllegalArgumentException.class,
+        "Cannot add field id as an identifier field: not a required field",
+        () -> new Schema(testSchema.asStruct().fields(), ImmutableSet.of(1)));
+
+    AssertHelpers.assertThrows("Creating schema with optional identifier field should fail",
+        IllegalArgumentException.class,
+        "Cannot add field float as an identifier field: must not be float or double field",
+        () -> new Schema(testSchema.asStruct().fields(), ImmutableSet.of(2)));
+
+    AssertHelpers.assertThrows("Creating schema with optional identifier field should fail",
+        IllegalArgumentException.class,
+        "Cannot add field double as an identifier field: must not be float or double field",
+        () -> new Schema(testSchema.asStruct().fields(), ImmutableSet.of(3)));
+
+    AssertHelpers.assertThrows("add a field with name not exist should fail",
+        IllegalArgumentException.class,
+        "not found in current schema or added columns",
+        () -> new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields("unknown")
+            .apply());
+
+    AssertHelpers.assertThrows("add a field of non-primitive type should fail",
+        IllegalArgumentException.class,
+        "not a primitive type field",
+        () -> new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields("locations")
+            .apply());
+
+    AssertHelpers.assertThrows("add an optional field should fail",
+        IllegalArgumentException.class,
+        "not a required field",
+        () -> new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields("data")
+            .apply());
+
+    AssertHelpers.assertThrows("add a map key nested field should fail",
+        IllegalArgumentException.class,
+        "must not be nested in " + SCHEMA.findField("locations"),
+        () -> new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields("locations.key.zip")
+            .apply());
+
+    AssertHelpers.assertThrows("add a nested field in list should fail",
+        IllegalArgumentException.class,
+        "must not be nested in " + SCHEMA.findField("points"),
+        () -> new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields("points.element.x")
+            .apply());
+
+    Schema newSchema = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .allowIncompatibleChanges()
+        .addRequiredColumn("col_float", Types.FloatType.get())
+        .addRequiredColumn("col_double", Types.DoubleType.get())
+        .addRequiredColumn("new", Types.StructType.of(
+            Types.NestedField.required(SCHEMA_LAST_COLUMN_ID + 3, "fields", Types.ListType.ofOptional(
+                SCHEMA_LAST_COLUMN_ID + 4, Types.StructType.of(
+                    Types.NestedField.required(SCHEMA_LAST_COLUMN_ID + 5, "nested", Types.StringType.get())
+                ))
+            )
+        ))
+        .addRequiredColumn("new_map", Types.MapType.ofRequired(SCHEMA_LAST_COLUMN_ID + 6, 11,
+            Types.StructType.of(
+                required(SCHEMA_LAST_COLUMN_ID + 7, "key_col", Types.StringType.get())
+            ),
+            Types.StructType.of(
+                required(SCHEMA_LAST_COLUMN_ID + 8, "val_col", Types.StringType.get())
+            )), "map of address to coordinate")
+        .apply();
+
+    int lastColId = SCHEMA_LAST_COLUMN_ID + 8;
+
+    AssertHelpers.assertThrows("add a double field should fail",
+        IllegalArgumentException.class,
+        "must not be float or double field",
+        () -> new SchemaUpdate(newSchema, lastColId)
+            .setIdentifierFields("col_double")
+            .apply());
+
+    AssertHelpers.assertThrows("add a float field should fail",
+        IllegalArgumentException.class,
+        "must not be float or double field",
+        () -> new SchemaUpdate(newSchema, lastColId)
+            .setIdentifierFields("col_float")
+            .apply());
+
+    AssertHelpers.assertThrows("add a map value nested field should fail",
+        IllegalArgumentException.class,
+        "must not be nested in " + newSchema.findField("new_map"),
+        () -> new SchemaUpdate(newSchema, lastColId)
+            .setIdentifierFields("new_map.value.val_col")
+            .apply());
+
+    AssertHelpers.assertThrows("add a nested field in struct of a map should fail",
+        IllegalArgumentException.class,
+        "must not be nested in " + newSchema.findField("new.fields"),
+        () -> new SchemaUpdate(newSchema, lastColId)
+            .setIdentifierFields("new.fields.element.nested")
+            .apply());
+  }
+
+  @Test
+  public void testDeleteIdentifierFieldColumns() {
+    Schema schemaWithIdentifierFields = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("id")
+        .apply();
+
+    Assert.assertEquals("delete column and then reset identifier field should succeed",
+        Sets.newHashSet(),
+        new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+            .deleteColumn("id").setIdentifierFields(Sets.newHashSet()).apply()
+            .identifierFieldIds());
+
+    Assert.assertEquals("delete reset identifier field and then delete column should succeed",
+        Sets.newHashSet(),
+        new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+            .setIdentifierFields(Sets.newHashSet()).deleteColumn("id").apply()
+            .identifierFieldIds());
+  }
+
+  @Test
+  public void testDeleteIdentifierFieldColumnsFails() {
+    Schema schemaWithIdentifierFields = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("id")
+        .apply();
+
+    AssertHelpers.assertThrows("delete an identifier column without setting identifier fields should fail",
+        IllegalArgumentException.class,
+        "Cannot delete identifier field 1: id: required int. To force deletion, " +
+            "also call setIdentifierFields to update identifier fields.",
+        () -> new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID).deleteColumn("id").apply());
+  }
+
+  @Test
+  public void testRenameIdentifierFields() {
+    Schema schemaWithIdentifierFields = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("id")
+        .apply();
+
+    Schema newSchema = new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+        .renameColumn("id", "id2")
+        .apply();
+
+    Assert.assertEquals("rename should not affect identifier fields",
+        Sets.newHashSet(SCHEMA.findField("id").fieldId()),
+        newSchema.identifierFieldIds());
+  }
+
+  @Test
+  public void testMoveIdentifierFields() {
+    Schema schemaWithIdentifierFields = new SchemaUpdate(SCHEMA, SCHEMA_LAST_COLUMN_ID)
+        .setIdentifierFields("id")
+        .apply();
+
+    Schema newSchema = new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+        .moveAfter("id", "locations")
+        .apply();
+
+    Assert.assertEquals("move after should not affect identifier fields",
+        Sets.newHashSet(SCHEMA.findField("id").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+        .moveBefore("id", "locations")
+        .apply();
+
+    Assert.assertEquals("move before should not affect identifier fields",
+        Sets.newHashSet(SCHEMA.findField("id").fieldId()),
+        newSchema.identifierFieldIds());
+
+    newSchema = new SchemaUpdate(schemaWithIdentifierFields, SCHEMA_LAST_COLUMN_ID)
+        .moveFirst("id")
+        .apply();
+
+    Assert.assertEquals("move first should not affect identifier fields",
+        Sets.newHashSet(SCHEMA.findField("id").fieldId()),
+        newSchema.identifierFieldIds());
+  }
 }
