@@ -59,11 +59,12 @@ import static org.apache.iceberg.TableMetadataParser.PARTITION_SPEC;
 import static org.apache.iceberg.TableMetadataParser.PROPERTIES;
 import static org.apache.iceberg.TableMetadataParser.SCHEMA;
 import static org.apache.iceberg.TableMetadataParser.SNAPSHOTS;
+import static org.apache.iceberg.TestHelpers.assertSameSchemaList;
 
 public class TestTableMetadata {
   private static final String TEST_LOCATION = "s3://bucket/test/location";
 
-  private static final Schema TEST_SCHEMA = new Schema(
+  private static final Schema TEST_SCHEMA = new Schema(7,
       Types.NestedField.required(1, "x", Types.LongType.get()),
       Types.NestedField.required(2, "y", Types.LongType.get(), "comment"),
       Types.NestedField.required(3, "z", Types.LongType.get())
@@ -88,11 +89,11 @@ public class TestTableMetadata {
   public void testJsonConversion() throws Exception {
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), SPEC_5.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, 7, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), SPEC_5.specId())));
 
     List<HistoryEntry> snapshotLog = ImmutableList.<HistoryEntry>builder()
@@ -100,8 +101,13 @@ public class TestTableMetadata {
         .add(new SnapshotLogEntry(currentSnapshot.timestampMillis(), currentSnapshot.snapshotId()))
         .build();
 
+    Schema schema = new Schema(6,
+        Types.NestedField.required(10, "x", Types.StringType.get()));
+
     TableMetadata expected = new TableMetadata(null, 2, UUID.randomUUID().toString(), TEST_LOCATION,
-        SEQ_NO, System.currentTimeMillis(), 3, TEST_SCHEMA, 5, ImmutableList.of(SPEC_5),
+        SEQ_NO, System.currentTimeMillis(), 3,
+        7, ImmutableList.of(TEST_SCHEMA, schema),
+        5, ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(),
         3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of("property", "value"), currentSnapshotId,
         Arrays.asList(previousSnapshot, currentSnapshot), snapshotLog, ImmutableList.of());
 
@@ -119,14 +125,23 @@ public class TestTableMetadata {
         expected.lastSequenceNumber(), metadata.lastSequenceNumber());
     Assert.assertEquals("Last column ID should match",
         expected.lastColumnId(), metadata.lastColumnId());
-    Assert.assertEquals("Schema should match",
-        expected.schema().asStruct(), metadata.schema().asStruct());
+    Assert.assertEquals("Current schema id should match",
+        expected.currentSchemaId(), metadata.currentSchemaId());
+    assertSameSchemaList(expected.schemas(), metadata.schemas());
     Assert.assertEquals("Partition spec should match",
         expected.spec().toString(), metadata.spec().toString());
     Assert.assertEquals("Default spec ID should match",
         expected.defaultSpecId(), metadata.defaultSpecId());
     Assert.assertEquals("PartitionSpec map should match",
         expected.specs(), metadata.specs());
+    Assert.assertEquals("lastAssignedFieldId across all PartitionSpecs should match",
+        expected.spec().lastAssignedFieldId(), metadata.lastAssignedPartitionId());
+    Assert.assertEquals("Default sort ID should match",
+        expected.defaultSortOrderId(), metadata.defaultSortOrderId());
+    Assert.assertEquals("Sort order should match",
+        expected.sortOrder(), metadata.sortOrder());
+    Assert.assertEquals("Sort order map should match",
+        expected.sortOrders(), metadata.sortOrders());
     Assert.assertEquals("Properties should match",
         expected.properties(), metadata.properties());
     Assert.assertEquals("Snapshot logs should match",
@@ -137,33 +152,39 @@ public class TestTableMetadata {
         (Long) previousSnapshotId, metadata.currentSnapshot().parentId());
     Assert.assertEquals("Current snapshot files should match",
         currentSnapshot.allManifests(), metadata.currentSnapshot().allManifests());
+    Assert.assertEquals("Schema ID for current snapshot should match",
+        (Integer) 7, metadata.currentSnapshot().schemaId());
     Assert.assertEquals("Previous snapshot ID should match",
         previousSnapshotId, metadata.snapshot(previousSnapshotId).snapshotId());
     Assert.assertEquals("Previous snapshot files should match",
         previousSnapshot.allManifests(),
         metadata.snapshot(previousSnapshotId).allManifests());
+    Assert.assertNull("Previous snapshot's schema ID should be null",
+        metadata.snapshot(previousSnapshotId).schemaId());
   }
 
   @Test
   public void testBackwardCompat() throws Exception {
     PartitionSpec spec = PartitionSpec.builderFor(TEST_SCHEMA).identity("x").withSpecId(6).build();
     SortOrder sortOrder = SortOrder.unsorted();
+    Schema schema = new Schema(TableMetadata.INITIAL_SCHEMA_ID, TEST_SCHEMA.columns());
 
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), spec.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), spec.specId())));
 
     TableMetadata expected = new TableMetadata(null, 1, null, TEST_LOCATION,
-        0, System.currentTimeMillis(), 3, TEST_SCHEMA, 6, ImmutableList.of(spec),
+        0, System.currentTimeMillis(), 3, TableMetadata.INITIAL_SCHEMA_ID,
+        ImmutableList.of(schema), 6, ImmutableList.of(spec), spec.lastAssignedFieldId(),
         TableMetadata.INITIAL_SORT_ORDER_ID, ImmutableList.of(sortOrder), ImmutableMap.of("property", "value"),
         currentSnapshotId, Arrays.asList(previousSnapshot, currentSnapshot), ImmutableList.of(), ImmutableList.of());
 
-    String asJson = toJsonWithoutSpecList(expected);
+    String asJson = toJsonWithoutSpecAndSchemaList(expected);
     TableMetadata metadata = TableMetadataParser
         .fromJson(ops.io(), null, JsonUtil.mapper().readValue(asJson, JsonNode.class));
 
@@ -176,8 +197,12 @@ public class TestTableMetadata {
         expected.lastSequenceNumber(), metadata.lastSequenceNumber());
     Assert.assertEquals("Last column ID should match",
         expected.lastColumnId(), metadata.lastColumnId());
-    Assert.assertEquals("Schema should match",
-        expected.schema().asStruct(), metadata.schema().asStruct());
+    Assert.assertEquals("Current schema ID should be default to TableMetadata.INITIAL_SCHEMA_ID",
+        TableMetadata.INITIAL_SCHEMA_ID, metadata.currentSchemaId());
+    Assert.assertEquals("Schemas size should match",
+        1, metadata.schemas().size());
+    Assert.assertEquals("Schemas should contain the schema",
+        metadata.schemas().get(0).asStruct(), schema.asStruct());
     Assert.assertEquals("Partition spec should be the default",
         expected.spec().toString(), metadata.spec().toString());
     Assert.assertEquals("Default spec ID should default to TableMetadata.INITIAL_SPEC_ID",
@@ -188,6 +213,8 @@ public class TestTableMetadata {
         metadata.specs().get(0).compatibleWith(spec));
     Assert.assertEquals("PartitionSpec should have ID TableMetadata.INITIAL_SPEC_ID",
         TableMetadata.INITIAL_SPEC_ID, metadata.specs().get(0).specId());
+    Assert.assertEquals("lastAssignedFieldId across all PartitionSpecs should match",
+        expected.spec().lastAssignedFieldId(), metadata.lastAssignedPartitionId());
     Assert.assertEquals("Properties should match",
         expected.properties(), metadata.properties());
     Assert.assertEquals("Snapshot logs should match",
@@ -198,6 +225,8 @@ public class TestTableMetadata {
         (Long) previousSnapshotId, metadata.currentSnapshot().parentId());
     Assert.assertEquals("Current snapshot files should match",
         currentSnapshot.allManifests(), metadata.currentSnapshot().allManifests());
+    Assert.assertNull("Current snapshot's schema ID should be null",
+        metadata.currentSnapshot().schemaId());
     Assert.assertEquals("Previous snapshot ID should match",
         previousSnapshotId, metadata.snapshot(previousSnapshotId).snapshotId());
     Assert.assertEquals("Previous snapshot files should match",
@@ -205,9 +234,11 @@ public class TestTableMetadata {
         metadata.snapshot(previousSnapshotId).allManifests());
     Assert.assertEquals("Snapshot logs should match",
             expected.previousFiles(), metadata.previousFiles());
+    Assert.assertNull("Previous snapshot's schema ID should be null",
+        metadata.snapshot(previousSnapshotId).schemaId());
   }
 
-  public static String toJsonWithoutSpecList(TableMetadata metadata) {
+  private static String toJsonWithoutSpecAndSchemaList(TableMetadata metadata) {
     StringWriter writer = new StringWriter();
     try {
       JsonGenerator generator = JsonUtil.factory().createGenerator(writer);
@@ -219,8 +250,9 @@ public class TestTableMetadata {
       generator.writeNumberField(LAST_UPDATED_MILLIS, metadata.lastUpdatedMillis());
       generator.writeNumberField(LAST_COLUMN_ID, metadata.lastColumnId());
 
+      // mimic an old writer by writing only schema and not the current ID or schema list
       generator.writeFieldName(SCHEMA);
-      SchemaParser.toJson(metadata.schema(), generator);
+      SchemaParser.toJson(metadata.schema().asStruct(), generator);
 
       // mimic an old writer by writing only partition-spec and not the default ID or spec list
       generator.writeFieldName(PARTITION_SPEC);
@@ -255,11 +287,11 @@ public class TestTableMetadata {
   public void testJsonWithPreviousMetadataLog() throws Exception {
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), SPEC_5.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), SPEC_5.specId())));
 
     List<HistoryEntry> reversedSnapshotLog = Lists.newArrayList();
@@ -269,7 +301,8 @@ public class TestTableMetadata {
         "/tmp/000001-" + UUID.randomUUID().toString() + ".metadata.json"));
 
     TableMetadata base = new TableMetadata(null, 1, UUID.randomUUID().toString(), TEST_LOCATION,
-        0, System.currentTimeMillis(), 3, TEST_SCHEMA, 5, ImmutableList.of(SPEC_5),
+        0, System.currentTimeMillis(), 3,
+        7, ImmutableList.of(TEST_SCHEMA), 5, ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(),
         3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of("property", "value"), currentSnapshotId,
         Arrays.asList(previousSnapshot, currentSnapshot), reversedSnapshotLog,
         ImmutableList.copyOf(previousMetadataLog));
@@ -285,11 +318,11 @@ public class TestTableMetadata {
   public void testAddPreviousMetadataRemoveNone() {
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), SPEC_5.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), SPEC_5.specId())));
 
     List<HistoryEntry> reversedSnapshotLog = Lists.newArrayList();
@@ -304,7 +337,8 @@ public class TestTableMetadata {
         "/tmp/000003-" + UUID.randomUUID().toString() + ".metadata.json");
 
     TableMetadata base = new TableMetadata(localInput(latestPreviousMetadata.file()), 1, UUID.randomUUID().toString(),
-        TEST_LOCATION, 0, currentTimestamp - 80, 3, TEST_SCHEMA, 5, ImmutableList.of(SPEC_5),
+        TEST_LOCATION, 0, currentTimestamp - 80, 3,
+        7, ImmutableList.of(TEST_SCHEMA), 5, ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(),
         3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of("property", "value"), currentSnapshotId,
         Arrays.asList(previousSnapshot, currentSnapshot), reversedSnapshotLog,
         ImmutableList.copyOf(previousMetadataLog));
@@ -324,11 +358,11 @@ public class TestTableMetadata {
   public void testAddPreviousMetadataRemoveOne() {
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), SPEC_5.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), SPEC_5.specId())));
 
     List<HistoryEntry> reversedSnapshotLog = Lists.newArrayList();
@@ -349,8 +383,9 @@ public class TestTableMetadata {
         "/tmp/000006-" + UUID.randomUUID().toString() + ".metadata.json");
 
     TableMetadata base = new TableMetadata(localInput(latestPreviousMetadata.file()), 1, UUID.randomUUID().toString(),
-        TEST_LOCATION, 0, currentTimestamp - 50, 3, TEST_SCHEMA, 5,
-        ImmutableList.of(SPEC_5), 3, ImmutableList.of(SORT_ORDER_3),
+        TEST_LOCATION, 0, currentTimestamp - 50, 3,
+        7, ImmutableList.of(TEST_SCHEMA), 5,
+        ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(), 3, ImmutableList.of(SORT_ORDER_3),
         ImmutableMap.of("property", "value"), currentSnapshotId,
         Arrays.asList(previousSnapshot, currentSnapshot), reversedSnapshotLog,
         ImmutableList.copyOf(previousMetadataLog));
@@ -375,11 +410,11 @@ public class TestTableMetadata {
   public void testAddPreviousMetadataRemoveMultiple() {
     long previousSnapshotId = System.currentTimeMillis() - new Random(1234).nextInt(3600);
     Snapshot previousSnapshot = new BaseSnapshot(
-        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, ImmutableList.of(
+        ops.io(), previousSnapshotId, null, previousSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.1.avro"), SPEC_5.specId())));
     long currentSnapshotId = System.currentTimeMillis();
     Snapshot currentSnapshot = new BaseSnapshot(
-        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, ImmutableList.of(
+        ops.io(), currentSnapshotId, previousSnapshotId, currentSnapshotId, null, null, null, ImmutableList.of(
         new GenericManifestFile(localInput("file:/tmp/manfiest.2.avro"), SPEC_5.specId())));
 
     List<HistoryEntry> reversedSnapshotLog = Lists.newArrayList();
@@ -400,8 +435,9 @@ public class TestTableMetadata {
         "/tmp/000006-" + UUID.randomUUID().toString() + ".metadata.json");
 
     TableMetadata base = new TableMetadata(localInput(latestPreviousMetadata.file()), 1, UUID.randomUUID().toString(),
-        TEST_LOCATION, 0, currentTimestamp - 50, 3, TEST_SCHEMA, 2,
-        ImmutableList.of(SPEC_5), TableMetadata.INITIAL_SORT_ORDER_ID, ImmutableList.of(SortOrder.unsorted()),
+        TEST_LOCATION, 0, currentTimestamp - 50, 3, 7, ImmutableList.of(TEST_SCHEMA), 2,
+        ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(),
+        TableMetadata.INITIAL_SORT_ORDER_ID, ImmutableList.of(SortOrder.unsorted()),
         ImmutableMap.of("property", "value"), currentSnapshotId,
         Arrays.asList(previousSnapshot, currentSnapshot), reversedSnapshotLog,
         ImmutableList.copyOf(previousMetadataLog));
@@ -427,7 +463,8 @@ public class TestTableMetadata {
     AssertHelpers.assertThrows("Should reject v2 metadata without a UUID",
         IllegalArgumentException.class, "UUID is required in format v2",
         () -> new TableMetadata(null, 2, null, TEST_LOCATION, SEQ_NO, System.currentTimeMillis(),
-            LAST_ASSIGNED_COLUMN_ID, TEST_SCHEMA, SPEC_5.specId(), ImmutableList.of(SPEC_5),
+            LAST_ASSIGNED_COLUMN_ID, 7, ImmutableList.of(TEST_SCHEMA),
+            SPEC_5.specId(), ImmutableList.of(SPEC_5), SPEC_5.lastAssignedFieldId(),
             3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of(), -1L,
             ImmutableList.of(), ImmutableList.of(), ImmutableList.of())
     );
@@ -439,8 +476,9 @@ public class TestTableMetadata {
     AssertHelpers.assertThrows("Should reject unsupported metadata",
         IllegalArgumentException.class, "Unsupported format version: v" + unsupportedVersion,
         () -> new TableMetadata(null, unsupportedVersion, null, TEST_LOCATION, SEQ_NO,
-            System.currentTimeMillis(), LAST_ASSIGNED_COLUMN_ID, TEST_SCHEMA, SPEC_5.specId(), ImmutableList.of(SPEC_5),
-            3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of(), -1L,
+            System.currentTimeMillis(), LAST_ASSIGNED_COLUMN_ID,
+            7, ImmutableList.of(TEST_SCHEMA), SPEC_5.specId(), ImmutableList.of(SPEC_5),
+            SPEC_5.lastAssignedFieldId(), 3, ImmutableList.of(SORT_ORDER_3), ImmutableMap.of(), -1L,
             ImmutableList.of(), ImmutableList.of(), ImmutableList.of())
     );
   }
@@ -476,6 +514,16 @@ public class TestTableMetadata {
     );
   }
 
+  @Test
+  public void testParserV2LastAssignedFieldIdValidation() throws Exception {
+    String unsupportedVersion = readTableMetadataInputFile("TableMetadataV2MissingLastPartitionId.json");
+    AssertHelpers.assertThrows("Should reject v2 metadata without last assigned partition field id",
+        IllegalArgumentException.class, "last-partition-id must exist in format v2",
+        () -> TableMetadataParser.fromJson(
+            ops.io(), null, JsonUtil.mapper().readValue(unsupportedVersion, JsonNode.class))
+    );
+  }
+
   @Test
   public void testParserV2SortOrderValidation() throws Exception {
     String unsupportedVersion = readTableMetadataInputFile("TableMetadataV2MissingSortOrder.json");
@@ -486,6 +534,26 @@ public class TestTableMetadata {
     );
   }
 
+  @Test
+  public void testParserV2CurrentSchemaIdValidation() throws Exception {
+    String unsupported = readTableMetadataInputFile("TableMetadataV2CurrentSchemaNotFound.json");
+    AssertHelpers.assertThrows("Should reject v2 metadata without valid schema id",
+        IllegalArgumentException.class, "Cannot find schema with current-schema-id=2 from schemas",
+        () -> TableMetadataParser.fromJson(
+            ops.io(), null, JsonUtil.mapper().readValue(unsupported, JsonNode.class))
+    );
+  }
+
+  @Test
+  public void testParserV2SchemasValidation() throws Exception {
+    String unsupported = readTableMetadataInputFile("TableMetadataV2MissingSchemas.json");
+    AssertHelpers.assertThrows("Should reject v2 metadata without schemas",
+        IllegalArgumentException.class, "schemas must exist in format v2",
+        () -> TableMetadataParser.fromJson(
+            ops.io(), null, JsonUtil.mapper().readValue(unsupported, JsonNode.class))
+    );
+  }
+
   private String readTableMetadataInputFile(String fileName) throws Exception {
     Path path = Paths.get(getClass().getClassLoader().getResource(fileName).toURI());
     return String.join("", java.nio.file.Files.readAllLines(path));
@@ -532,6 +600,76 @@ public class TestTableMetadata {
         () -> metadata.updatePartitionSpec(spec));
   }
 
+  @Test
+  public void testBuildReplacementForV1Table() {
+    Schema schema = new Schema(
+        Types.NestedField.required(1, "x", Types.LongType.get()),
+        Types.NestedField.required(2, "y", Types.LongType.get())
+    );
+    PartitionSpec spec = PartitionSpec.builderFor(schema).withSpecId(0)
+        .identity("x")
+        .identity("y")
+        .build();
+    String location = "file://tmp/db/table";
+    TableMetadata metadata = TableMetadata.newTableMetadata(
+        schema, spec, SortOrder.unsorted(), location, ImmutableMap.of(), 1);
+    Assert.assertEquals(spec, metadata.spec());
+
+    Schema updatedSchema = new Schema(
+        Types.NestedField.required(1, "x", Types.LongType.get()),
+        Types.NestedField.required(2, "z", Types.StringType.get()),
+        Types.NestedField.required(3, "y", Types.LongType.get())
+    );
+    PartitionSpec updatedSpec = PartitionSpec.builderFor(updatedSchema).withSpecId(0)
+        .bucket("z", 8)
+        .identity("x")
+        .build();
+    TableMetadata updated = metadata.buildReplacement(
+        updatedSchema, updatedSpec, SortOrder.unsorted(), location, ImmutableMap.of());
+    PartitionSpec expected = PartitionSpec.builderFor(updated.schema()).withSpecId(1)
+        .add(1, 1000, "x", "identity")
+        .add(2, 1001, "y", "void")
+        .add(3, 1002, "z_bucket", "bucket[8]")
+        .build();
+    Assert.assertEquals(
+        "Should reassign the partition field IDs and reuse any existing IDs for equivalent fields",
+        expected, updated.spec());
+  }
+
+  @Test
+  public void testBuildReplacementForV2Table() {
+    Schema schema = new Schema(
+        Types.NestedField.required(1, "x", Types.LongType.get()),
+        Types.NestedField.required(2, "y", Types.LongType.get())
+    );
+    PartitionSpec spec = PartitionSpec.builderFor(schema).withSpecId(0)
+        .identity("x")
+        .identity("y")
+        .build();
+    String location = "file://tmp/db/table";
+    TableMetadata metadata = TableMetadata.newTableMetadata(
+        schema, spec, SortOrder.unsorted(), location, ImmutableMap.of(), 2);
+    Assert.assertEquals(spec, metadata.spec());
+
+    Schema updatedSchema = new Schema(
+        Types.NestedField.required(1, "x", Types.LongType.get()),
+        Types.NestedField.required(2, "z", Types.StringType.get())
+    );
+    PartitionSpec updatedSpec = PartitionSpec.builderFor(updatedSchema).withSpecId(0)
+        .bucket("z", 8)
+        .identity("x")
+        .build();
+    TableMetadata updated = metadata.buildReplacement(
+        updatedSchema, updatedSpec, SortOrder.unsorted(), location, ImmutableMap.of());
+    PartitionSpec expected = PartitionSpec.builderFor(updated.schema()).withSpecId(1)
+        .add(3, 1002, "z_bucket", "bucket[8]")
+        .add(1, 1000, "x", "identity")
+        .build();
+    Assert.assertEquals(
+        "Should reassign the partition field IDs and reuse any existing IDs for equivalent fields",
+        expected, updated.spec());
+  }
+
   @Test
   public void testSortOrder() {
     Schema schema = new Schema(
@@ -584,4 +722,172 @@ public class TestTableMetadata {
     Assert.assertEquals("Should be nulls first",
         NullOrder.NULLS_FIRST, sortedByX.sortOrder().fields().get(0).nullOrder());
   }
+
+  @Test
+  public void testParseSchemaIdentifierFields() throws Exception {
+    String data = readTableMetadataInputFile("TableMetadataV2Valid.json");
+    TableMetadata parsed = TableMetadataParser.fromJson(
+        ops.io(), null, JsonUtil.mapper().readValue(data, JsonNode.class));
+    Assert.assertEquals(Sets.newHashSet(), parsed.schemasById().get(0).identifierFieldIds());
+    Assert.assertEquals(Sets.newHashSet(1, 2), parsed.schemasById().get(1).identifierFieldIds());
+  }
+
+  @Test
+  public void testUpdateSchemaIdentifierFields() {
+    Schema schema = new Schema(
+        Types.NestedField.required(10, "x", Types.StringType.get())
+    );
+
+    TableMetadata meta = TableMetadata.newTableMetadata(
+        schema, PartitionSpec.unpartitioned(), null, ImmutableMap.of());
+
+    Schema newSchema = new Schema(
+        Lists.newArrayList(Types.NestedField.required(1, "x", Types.StringType.get())),
+        Sets.newHashSet(1)
+    );
+    TableMetadata newMeta = meta.updateSchema(newSchema, 1);
+    Assert.assertEquals(2, newMeta.schemas().size());
+    Assert.assertEquals(Sets.newHashSet(1), newMeta.schema().identifierFieldIds());
+  }
+
+  @Test
+  public void testUpdateSchema() {
+    Schema schema = new Schema(0,
+        Types.NestedField.required(1, "y", Types.LongType.get(), "comment")
+    );
+    TableMetadata freshTable = TableMetadata.newTableMetadata(
+        schema, PartitionSpec.unpartitioned(), null, ImmutableMap.of());
+    Assert.assertEquals("Should use TableMetadata.INITIAL_SCHEMA_ID for current schema id",
+        TableMetadata.INITIAL_SCHEMA_ID, freshTable.currentSchemaId());
+    assertSameSchemaList(ImmutableList.of(schema), freshTable.schemas());
+    Assert.assertEquals("Should have expected schema upon return",
+        schema.asStruct(), freshTable.schema().asStruct());
+    Assert.assertEquals("Should return expected last column id", 1, freshTable.lastColumnId());
+
+    // update schema
+    Schema schema2 = new Schema(
+        Types.NestedField.required(1, "y", Types.LongType.get(), "comment"),
+        Types.NestedField.required(2, "x", Types.StringType.get())
+    );
+    TableMetadata twoSchemasTable = freshTable.updateSchema(schema2, 2);
+    Assert.assertEquals("Should have current schema id as 1",
+        1, twoSchemasTable.currentSchemaId());
+    assertSameSchemaList(ImmutableList.of(schema, new Schema(1, schema2.columns())),
+        twoSchemasTable.schemas());
+    Assert.assertEquals("Should have expected schema upon return",
+        schema2.asStruct(), twoSchemasTable.schema().asStruct());
+    Assert.assertEquals("Should return expected last column id", 2, twoSchemasTable.lastColumnId());
+
+    // update schema with the the same schema and last column ID as current shouldn't cause change
+    Schema sameSchema2 = new Schema(
+        Types.NestedField.required(1, "y", Types.LongType.get(), "comment"),
+        Types.NestedField.required(2, "x", Types.StringType.get())
+    );
+    TableMetadata sameSchemaTable = twoSchemasTable.updateSchema(sameSchema2, 2);
+    Assert.assertSame("Should return same table metadata",
+        twoSchemasTable, sameSchemaTable);
+
+    // update schema with the the same schema and different last column ID as current should create a new table
+    TableMetadata differentColumnIdTable = sameSchemaTable.updateSchema(sameSchema2, 3);
+    Assert.assertEquals("Should have current schema id as 1",
+        1, differentColumnIdTable.currentSchemaId());
+    assertSameSchemaList(ImmutableList.of(schema, new Schema(1, schema2.columns())),
+        differentColumnIdTable.schemas());
+    Assert.assertEquals("Should have expected schema upon return",
+        schema2.asStruct(), differentColumnIdTable.schema().asStruct());
+    Assert.assertEquals("Should return expected last column id",
+        3, differentColumnIdTable.lastColumnId());
+
+    // update schema with old schema does not change schemas
+    TableMetadata revertSchemaTable = differentColumnIdTable.updateSchema(schema, 3);
+    Assert.assertEquals("Should have current schema id as 0",
+        0, revertSchemaTable.currentSchemaId());
+    assertSameSchemaList(ImmutableList.of(schema, new Schema(1, schema2.columns())),
+        revertSchemaTable.schemas());
+    Assert.assertEquals("Should have expected schema upon return",
+        schema.asStruct(), revertSchemaTable.schema().asStruct());
+    Assert.assertEquals("Should return expected last column id",
+        3, revertSchemaTable.lastColumnId());
+
+    // create new schema will use the largest schema id + 1
+    Schema schema3 = new Schema(
+        Types.NestedField.required(2, "y", Types.LongType.get(), "comment"),
+        Types.NestedField.required(4, "x", Types.StringType.get()),
+        Types.NestedField.required(6, "z", Types.IntegerType.get())
+    );
+    TableMetadata threeSchemaTable = revertSchemaTable.updateSchema(schema3, 6);
+    Assert.assertEquals("Should have current schema id as 2",
+        2, threeSchemaTable.currentSchemaId());
+    assertSameSchemaList(ImmutableList.of(schema,
+            new Schema(1, schema2.columns()),
+            new Schema(2, schema3.columns())), threeSchemaTable.schemas());
+    Assert.assertEquals("Should have expected schema upon return",
+        schema3.asStruct(), threeSchemaTable.schema().asStruct());
+    Assert.assertEquals("Should return expected last column id",
+        6, threeSchemaTable.lastColumnId());
+  }
+
+  @Test
+  public void testCreateV2MetadataThroughTableProperty() {
+    Schema schema = new Schema(
+        Types.NestedField.required(10, "x", Types.StringType.get())
+    );
+
+    TableMetadata meta = TableMetadata.newTableMetadata(schema, PartitionSpec.unpartitioned(), null,
+        ImmutableMap.of(TableProperties.FORMAT_VERSION, "2", "key", "val"));
+
+    Assert.assertEquals("format version should be configured based on the format-version key",
+        2, meta.formatVersion());
+    Assert.assertEquals("should not contain format-version in properties",
+        ImmutableMap.of("key", "val"), meta.properties());
+  }
+
+  @Test
+  public void testReplaceV1MetadataToV2ThroughTableProperty() {
+    Schema schema = new Schema(
+        Types.NestedField.required(10, "x", Types.StringType.get())
+    );
+
+    TableMetadata meta = TableMetadata.newTableMetadata(schema, PartitionSpec.unpartitioned(), null,
+        ImmutableMap.of(TableProperties.FORMAT_VERSION, "1", "key", "val"));
+
+    meta = meta.buildReplacement(meta.schema(), meta.spec(), meta.sortOrder(), meta.location(),
+        ImmutableMap.of(TableProperties.FORMAT_VERSION, "2", "key2", "val2"));
+
+    Assert.assertEquals("format version should be configured based on the format-version key",
+        2, meta.formatVersion());
+    Assert.assertEquals("should not contain format-version but should contain old and new properties",
+        ImmutableMap.of("key", "val", "key2", "val2"), meta.properties());
+  }
+
+  @Test
+  public void testUpgradeV1MetadataToV2ThroughTableProperty() {
+    Schema schema = new Schema(
+        Types.NestedField.required(10, "x", Types.StringType.get())
+    );
+
+    TableMetadata meta = TableMetadata.newTableMetadata(schema, PartitionSpec.unpartitioned(), null,
+        ImmutableMap.of(TableProperties.FORMAT_VERSION, "1", "key", "val"));
+
+    meta = meta.replaceProperties(ImmutableMap.of(TableProperties.FORMAT_VERSION,
+        "2", "key2", "val2"));
+
+    Assert.assertEquals("format version should be configured based on the format-version key",
+        2, meta.formatVersion());
+    Assert.assertEquals("should not contain format-version but should contain new properties",
+        ImmutableMap.of("key2", "val2"), meta.properties());
+  }
+
+  @Test
+  public void testNoReservedPropertyForTableMetadataCreation() {
+    Schema schema = new Schema(
+        Types.NestedField.required(10, "x", Types.StringType.get())
+    );
+
+    AssertHelpers.assertThrows("should not allow reserved table property when creating table metadata",
+        IllegalArgumentException.class,
+        "Table properties should not contain reserved properties, but got {format-version=1}",
+        () -> TableMetadata.newTableMetadata(schema, PartitionSpec.unpartitioned(), null, "/tmp",
+            ImmutableMap.of(TableProperties.FORMAT_VERSION, "1"), 1));
+  }
 }
