@@ -88,8 +88,7 @@ public class TestTaskEqualityDeltaWriter extends TableTestBase {
     this.metadataDir = new File(tableDir, "metadata");
 
     this.table = create(SCHEMA, PartitionSpec.unpartitioned());
-    this.fileFactory = new OutputFileFactory(table.spec(), format, table.locationProvider(), table.io(),
-        table.encryption(), 1, 1);
+    this.fileFactory = OutputFileFactory.builderFor(table, 1, 1).format(format).build();
 
     this.idFieldId = table.schema().findField("id").fieldId();
     this.dataFieldId = table.schema().findField("data").fieldId();
