@@ -40,7 +40,6 @@ import org.apache.flink.streaming.api.operators.StreamOperatorParameters;
 import org.apache.flink.streaming.util.OneInputStreamOperatorTestHarness;
 import org.apache.flink.table.data.RowData;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.iceberg.AssertHelpers;
 import org.apache.iceberg.DataFile;
 import org.apache.iceberg.DeleteFile;
 import org.apache.iceberg.FileFormat;
@@ -49,9 +48,9 @@ import org.apache.iceberg.ManifestContent;
 import org.apache.iceberg.ManifestFile;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.TableTestBase;
-import org.apache.iceberg.exceptions.ValidationException;
 import org.apache.iceberg.flink.FlinkSchemaUtil;
 import org.apache.iceberg.flink.SimpleDataUtil;
+import org.apache.iceberg.flink.TestHelpers;
 import org.apache.iceberg.flink.TestTableLoader;
 import org.apache.iceberg.io.FileAppenderFactory;
 import org.apache.iceberg.io.WriteResult;
@@ -66,6 +65,7 @@ import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 
 import static org.apache.iceberg.TableProperties.DEFAULT_FILE_FORMAT;
+import static org.apache.iceberg.flink.sink.IcebergFilesCommitter.MAX_CONTINUOUS_EMPTY_COMMITS;
 import static org.apache.iceberg.flink.sink.ManifestOutputFileFactory.FLINK_MANIFEST_LOCATION;
 
 @RunWith(Parameterized.class)
@@ -109,6 +109,7 @@ public class TestIcebergFilesCommitter extends TableTestBase {
     table.updateProperties()
         .set(DEFAULT_FILE_FORMAT, format.name())
         .set(FLINK_MANIFEST_LOCATION, flinkManifestFolder.getAbsolutePath())
+        .set(MAX_CONTINUOUS_EMPTY_COMMITS, "1")
         .commit();
   }
 
@@ -140,6 +141,30 @@ public class TestIcebergFilesCommitter extends TableTestBase {
     }
   }
 
+  @Test
+  public void testMaxContinuousEmptyCommits() throws Exception {
+    table.updateProperties()
+        .set(MAX_CONTINUOUS_EMPTY_COMMITS, "3")
+        .commit();
+
+    JobID jobId = new JobID();
+    long checkpointId = 0;
+    long timestamp = 0;
+    try (OneInputStreamOperatorTestHarness<WriteResult, Void> harness = createStreamSink(jobId)) {
+      harness.setup();
+      harness.open();
+
+      assertSnapshotSize(0);
+
+      for (int i = 1; i <= 9; i++) {
+        harness.snapshot(++checkpointId, ++timestamp);
+        harness.notifyOfCompletedCheckpoint(checkpointId);
+
+        assertSnapshotSize(i / 3);
+      }
+    }
+  }
+
   private WriteResult of(DataFile dataFile) {
     return WriteResult.builder().addDataFiles(dataFile).build();
   }
@@ -578,7 +603,7 @@ public class TestIcebergFilesCommitter extends TableTestBase {
       // 2. Read the data files from manifests and assert.
       List<DataFile> dataFiles = FlinkManifestUtil.readDataFiles(createTestingManifestFile(manifestPath), table.io());
       Assert.assertEquals(1, dataFiles.size());
-      TestFlinkManifest.checkContentFile(dataFile1, dataFiles.get(0));
+      TestHelpers.assertEquals(dataFile1, dataFiles.get(0));
 
       // 3. notifyCheckpointComplete for checkpoint#1
       harness.notifyOfCompletedCheckpoint(checkpoint);
@@ -619,7 +644,7 @@ public class TestIcebergFilesCommitter extends TableTestBase {
       // 2. Read the data files from manifests and assert.
       List<DataFile> dataFiles = FlinkManifestUtil.readDataFiles(createTestingManifestFile(manifestPath), table.io());
       Assert.assertEquals(1, dataFiles.size());
-      TestFlinkManifest.checkContentFile(dataFile1, dataFiles.get(0));
+      TestHelpers.assertEquals(dataFile1, dataFiles.get(0));
 
       // 3. notifyCheckpointComplete for checkpoint#1
       harness.notifyOfCompletedCheckpoint(checkpoint);
@@ -652,66 +677,6 @@ public class TestIcebergFilesCommitter extends TableTestBase {
     }
   }
 
-  @Test
-  public void testValidateDataFileExist() throws Exception {
-    Assume.assumeFalse("Only support equality-delete in format v2.", formatVersion < 2);
-    long timestamp = 0;
-    long checkpoint = 10;
-    JobID jobId = new JobID();
-    FileAppenderFactory<RowData> appenderFactory = createDeletableAppenderFactory();
-
-    RowData insert1 = SimpleDataUtil.createInsert(1, "aaa");
-    DataFile dataFile1 = writeDataFile("data-file-1", ImmutableList.of(insert1));
-
-    try (OneInputStreamOperatorTestHarness<WriteResult, Void> harness = createStreamSink(jobId)) {
-      harness.setup();
-      harness.open();
-
-      // Txn#1: insert the row <1, 'aaa'>
-      harness.processElement(WriteResult.builder()
-              .addDataFiles(dataFile1)
-              .build(),
-          ++timestamp);
-      harness.snapshot(checkpoint, ++timestamp);
-      harness.notifyOfCompletedCheckpoint(checkpoint);
-
-      // Txn#2: Overwrite the committed data-file-1
-      RowData insert2 = SimpleDataUtil.createInsert(2, "bbb");
-      DataFile dataFile2 = writeDataFile("data-file-2", ImmutableList.of(insert2));
-      new TestTableLoader(tablePath)
-          .loadTable()
-          .newOverwrite()
-          .addFile(dataFile2)
-          .deleteFile(dataFile1)
-          .commit();
-    }
-
-    try (OneInputStreamOperatorTestHarness<WriteResult, Void> harness = createStreamSink(jobId)) {
-      harness.setup();
-      harness.open();
-
-      // Txn#3: position-delete the <1, 'aaa'> (NOT committed).
-      DeleteFile deleteFile1 = writePosDeleteFile(appenderFactory,
-          "pos-delete-file-1",
-          ImmutableList.of(Pair.of(dataFile1.path(), 0L)));
-      harness.processElement(WriteResult.builder()
-              .addDeleteFiles(deleteFile1)
-              .addReferencedDataFiles(dataFile1.path())
-              .build(),
-          ++timestamp);
-      harness.snapshot(++checkpoint, ++timestamp);
-
-      // Txn#3: validate will be failure when committing.
-      final long currentCheckpointId = checkpoint;
-      AssertHelpers.assertThrows("Validation should be failure because of non-exist data files.",
-          ValidationException.class, "Cannot commit, missing data files",
-          () -> {
-            harness.notifyOfCompletedCheckpoint(currentCheckpointId);
-            return null;
-          });
-    }
-  }
-
   @Test
   public void testCommitTwoCheckpointsInSingleTxn() throws Exception {
     Assume.assumeFalse("Only support equality-delete in format v2.", formatVersion < 2);
@@ -788,7 +753,7 @@ public class TestIcebergFilesCommitter extends TableTestBase {
 
   private ManifestFile createTestingManifestFile(Path manifestPath) {
     return new GenericManifestFile(manifestPath.toAbsolutePath().toString(), manifestPath.toFile().length(), 0,
-        ManifestContent.DATA, 0, 0, 0L, 0, 0, 0, 0, 0, 0, null);
+        ManifestContent.DATA, 0, 0, 0L, 0, 0, 0, 0, 0, 0, null, null);
   }
 
   private List<Path> assertFlinkManifests(int expectedCount) throws IOException {
