@@ -19,9 +19,15 @@
 
 package org.apache.iceberg.hive;
 
+import java.util.Collections;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.stream.IntStream;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hive.metastore.HiveMetaStoreClient;
+import org.apache.hadoop.hive.metastore.api.LockRequest;
 import org.apache.hadoop.hive.metastore.api.LockResponse;
 import org.apache.hadoop.hive.metastore.api.LockState;
 import org.apache.iceberg.AssertHelpers;
@@ -41,12 +47,17 @@ import static org.mockito.Matchers.any;
 import static org.mockito.Matchers.eq;
 import static org.mockito.Mockito.doNothing;
 import static org.mockito.Mockito.doReturn;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.reset;
 import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
 public class TestHiveCommitLocks extends HiveTableBaseTest {
   private static HiveTableOperations spyOps = null;
   private static HiveClientPool spyClientPool = null;
+  private static CachedClientPool spyCachedClientPool = null;
   private static Configuration overriddenHiveConf = new Configuration(hiveConf);
   private static AtomicReference<HiveMetaStoreClient> spyClientRef = new AtomicReference<>();
   private static HiveMetaStoreClient spyClient = null;
@@ -75,6 +86,10 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     });
 
     spyClientPool.run(HiveMetaStoreClient::isLocalMetaStore); // To ensure new client is created.
+
+    spyCachedClientPool = spy(new CachedClientPool(hiveConf, Collections.emptyMap()));
+    when(spyCachedClientPool.clientPool()).thenAnswer(invocation -> spyClientPool);
+
     Assert.assertNotNull(spyClientRef.get());
 
     spyClient = spyClientRef.get();
@@ -99,7 +114,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     Assert.assertEquals(2, ops.current().schema().columns().size());
 
-    spyOps = spy(new HiveTableOperations(overriddenHiveConf, spyClientPool, ops.io(), catalog.name(),
+    spyOps = spy(new HiveTableOperations(overriddenHiveConf, spyCachedClientPool, ops.io(), catalog.name(),
             dbName, tableName));
   }
 
@@ -204,4 +219,31 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
         "Could not acquire the lock on",
         () -> spyOps.doCommit(metadataV2, metadataV1));
   }
+
+  @Test
+  public void testTableLevelProcessLockBlocksConcurrentHMSRequestsForSameTable() throws Exception {
+    int numConcurrentCommits = 10;
+    // resetting the spy client to forget about prior call history
+    reset(spyClient);
+
+    // simulate several concurrent commit operations on the same table
+    ExecutorService executor = Executors.newFixedThreadPool(numConcurrentCommits);
+    IntStream.range(0, numConcurrentCommits).forEach(i ->
+        executor.submit(() -> {
+          try {
+            spyOps.doCommit(metadataV2, metadataV1);
+          } catch (CommitFailedException e) {
+            // failures are expected here when checking the base version
+            // it's no problem, we're not testing the actual commit success here, only the HMS lock acquisition attempts
+          }
+        }));
+    executor.shutdown();
+    executor.awaitTermination(30, TimeUnit.SECONDS);
+
+    // intra-process commits to the same table should be serialized now
+    // i.e. no thread should receive WAITING state from HMS and have to call checkLock periodically
+    verify(spyClient, never()).checkLock(any(Long.class));
+    // all threads eventually got their turn
+    verify(spyClient, times(numConcurrentCommits)).lock(any(LockRequest.class));
+  }
 }
