@@ -25,6 +25,8 @@ import java.util.Optional;
 import java.util.Properties;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.iceberg.AssertHelpers;
+import org.apache.iceberg.CatalogProperties;
+import org.apache.iceberg.CatalogUtil;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.PartitionSpecParser;
 import org.apache.iceberg.Schema;
@@ -32,12 +34,12 @@ import org.apache.iceberg.SchemaParser;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.catalog.Catalog;
 import org.apache.iceberg.catalog.TableIdentifier;
-import org.apache.iceberg.exceptions.NoSuchNamespaceException;
 import org.apache.iceberg.exceptions.NoSuchTableException;
 import org.apache.iceberg.hadoop.HadoopCatalog;
 import org.apache.iceberg.hadoop.HadoopTables;
 import org.apache.iceberg.hive.HiveCatalog;
 import org.apache.iceberg.types.Types;
+import org.assertj.core.api.Assertions;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Rule;
@@ -63,6 +65,7 @@ public class TestCatalogs {
 
   @Test
   public void testLoadTableFromLocation() throws IOException {
+    conf.set(InputFormatConfig.CATALOG, Catalogs.LOCATION);
     AssertHelpers.assertThrows(
             "Should complain about table location not set", IllegalArgumentException.class,
             "location not set", () -> Catalogs.loadTable(conf));
@@ -77,14 +80,15 @@ public class TestCatalogs {
 
   @Test
   public void testLoadTableFromCatalog() throws IOException {
-    conf.set("warehouse.location", temp.newFolder("hadoop", "warehouse").toString());
-    conf.setClass(InputFormatConfig.CATALOG_LOADER_CLASS, CustomHadoopCatalogLoader.class, CatalogLoader.class);
+    String defaultCatalogName = "default";
+    String warehouseLocation = temp.newFolder("hadoop", "warehouse").toString();
+    setCustomCatalogProperties(defaultCatalogName, warehouseLocation);
 
     AssertHelpers.assertThrows(
             "Should complain about table identifier not set", IllegalArgumentException.class,
             "identifier not set", () -> Catalogs.loadTable(conf));
 
-    HadoopCatalog catalog = new CustomHadoopCatalog(conf);
+    HadoopCatalog catalog = new CustomHadoopCatalog(conf, warehouseLocation);
     Table hadoopCatalogTable = catalog.createTable(TableIdentifier.of("table"), SCHEMA);
 
     conf.set(InputFormatConfig.TABLE_IDENTIFIER, "table");
@@ -100,6 +104,7 @@ public class TestCatalogs {
         "Should complain about table schema not set", NullPointerException.class,
         "schema not set", () -> Catalogs.createTable(conf, missingSchema));
 
+    conf.set(InputFormatConfig.CATALOG, Catalogs.LOCATION);
     Properties missingLocation = new Properties();
     missingLocation.put(InputFormatConfig.TABLE_SCHEMA, SchemaParser.toJson(SCHEMA));
     AssertHelpers.assertThrows(
@@ -138,18 +143,21 @@ public class TestCatalogs {
   @Test
   public void testCreateDropTableToCatalog() throws IOException {
     TableIdentifier identifier = TableIdentifier.of("test", "table");
+    String defaultCatalogName = "default";
+    String warehouseLocation = temp.newFolder("hadoop", "warehouse").toString();
 
-    conf.set("warehouse.location", temp.newFolder("hadoop", "warehouse").toString());
-    conf.setClass(InputFormatConfig.CATALOG_LOADER_CLASS, CustomHadoopCatalogLoader.class, CatalogLoader.class);
+    setCustomCatalogProperties(defaultCatalogName, warehouseLocation);
 
     Properties missingSchema = new Properties();
     missingSchema.put("name", identifier.toString());
+    missingSchema.put(InputFormatConfig.CATALOG_NAME, defaultCatalogName);
     AssertHelpers.assertThrows(
         "Should complain about table schema not set", NullPointerException.class,
         "schema not set", () -> Catalogs.createTable(conf, missingSchema));
 
     Properties missingIdentifier = new Properties();
     missingIdentifier.put(InputFormatConfig.TABLE_SCHEMA, SchemaParser.toJson(SCHEMA));
+    missingIdentifier.put(InputFormatConfig.CATALOG_NAME, defaultCatalogName);
     AssertHelpers.assertThrows(
         "Should complain about table identifier not set", NullPointerException.class,
         "identifier not set", () -> Catalogs.createTable(conf, missingIdentifier));
@@ -159,10 +167,11 @@ public class TestCatalogs {
     properties.put(InputFormatConfig.TABLE_SCHEMA, SchemaParser.toJson(SCHEMA));
     properties.put(InputFormatConfig.PARTITION_SPEC, PartitionSpecParser.toJson(SPEC));
     properties.put("dummy", "test");
+    properties.put(InputFormatConfig.CATALOG_NAME, defaultCatalogName);
 
     Catalogs.createTable(conf, properties);
 
-    HadoopCatalog catalog = new CustomHadoopCatalog(conf);
+    HadoopCatalog catalog = new CustomHadoopCatalog(conf, warehouseLocation);
     Table table = catalog.loadTable(identifier);
 
     Assert.assertEquals(SchemaParser.toJson(SCHEMA), SchemaParser.toJson(table.schema()));
@@ -175,6 +184,7 @@ public class TestCatalogs {
 
     Properties dropProperties = new Properties();
     dropProperties.put("name", identifier.toString());
+    dropProperties.put(InputFormatConfig.CATALOG_NAME, defaultCatalogName);
     Catalogs.dropTable(conf, dropProperties);
 
     AssertHelpers.assertThrows(
@@ -183,51 +193,158 @@ public class TestCatalogs {
   }
 
   @Test
-  public void testLoadCatalog() throws IOException {
-    Assert.assertFalse(Catalogs.loadCatalog(conf).isPresent());
-
-    conf.set(InputFormatConfig.CATALOG, "foo");
-    AssertHelpers.assertThrows(
-            "Should complain about catalog not supported", NoSuchNamespaceException.class,
-            "is not supported", () -> Catalogs.loadCatalog(conf));
-
-    conf.set(InputFormatConfig.CATALOG, "hadoop");
-    Optional<Catalog> hadoopCatalog = Catalogs.loadCatalog(conf);
-
-    Assert.assertTrue(hadoopCatalog.isPresent());
-    Assert.assertTrue(hadoopCatalog.get() instanceof HadoopCatalog);
-
-    conf.set(InputFormatConfig.CATALOG, "hive");
-    Optional<Catalog> hiveCatalog = Catalogs.loadCatalog(conf);
+  public void testLegacyLoadCatalogDefault() {
+    Optional<Catalog> defaultCatalog = Catalogs.loadCatalog(conf, null);
+    Assert.assertTrue(defaultCatalog.isPresent());
+    Assertions.assertThat(defaultCatalog.get()).isInstanceOf(HiveCatalog.class);
+    Assert.assertTrue(Catalogs.hiveCatalog(conf, new Properties()));
+  }
 
+  @Test
+  public void testLegacyLoadCatalogHive() {
+    conf.set(InputFormatConfig.CATALOG, CatalogUtil.ICEBERG_CATALOG_TYPE_HIVE);
+    Optional<Catalog> hiveCatalog = Catalogs.loadCatalog(conf, null);
     Assert.assertTrue(hiveCatalog.isPresent());
-    Assert.assertTrue(hiveCatalog.get() instanceof HiveCatalog);
+    Assertions.assertThat(hiveCatalog.get()).isInstanceOf(HiveCatalog.class);
+    Assert.assertTrue(Catalogs.hiveCatalog(conf, new Properties()));
+  }
 
-    conf.set("warehouse.location", temp.newFolder("hadoop", "warehouse").toString());
-    conf.setClass(InputFormatConfig.CATALOG_LOADER_CLASS, CustomHadoopCatalogLoader.class, CatalogLoader.class);
-    Optional<Catalog> customHadoopCatalog = Catalogs.loadCatalog(conf);
+  @Test
+  public void testLegacyLoadCatalogHadoop() {
+    conf.set(InputFormatConfig.CATALOG, CatalogUtil.ICEBERG_CATALOG_TYPE_HADOOP);
+    conf.set(InputFormatConfig.HADOOP_CATALOG_WAREHOUSE_LOCATION, "/tmp/mylocation");
+    Optional<Catalog> hadoopCatalog = Catalogs.loadCatalog(conf, null);
+    Assert.assertTrue(hadoopCatalog.isPresent());
+    Assertions.assertThat(hadoopCatalog.get()).isInstanceOf(HadoopCatalog.class);
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, new Properties()));
+  }
 
+  @Test
+  public void testLegacyLoadCatalogCustom() {
+    conf.set(InputFormatConfig.CATALOG_LOADER_CLASS, CustomHadoopCatalog.class.getName());
+    conf.set(InputFormatConfig.HADOOP_CATALOG_WAREHOUSE_LOCATION, "/tmp/mylocation");
+    Optional<Catalog> customHadoopCatalog = Catalogs.loadCatalog(conf, null);
     Assert.assertTrue(customHadoopCatalog.isPresent());
-    Assert.assertTrue(customHadoopCatalog.get() instanceof CustomHadoopCatalog);
+    Assertions.assertThat(customHadoopCatalog.get()).isInstanceOf(CustomHadoopCatalog.class);
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, new Properties()));
+  }
+
+  @Test
+  public void testLegacyLoadCatalogLocation() {
+    conf.set(InputFormatConfig.CATALOG, Catalogs.LOCATION);
+    Assert.assertFalse(Catalogs.loadCatalog(conf, null).isPresent());
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, new Properties()));
+  }
+
+  @Test
+  public void testLegacyLoadCatalogUnknown() {
+    conf.set(InputFormatConfig.CATALOG, "fooType");
+    AssertHelpers.assertThrows(
+            "should complain about catalog not supported", UnsupportedOperationException.class,
+            "Unknown catalog type", () -> Catalogs.loadCatalog(conf, null));
+  }
+
+  @Test
+  public void testLoadCatalogDefault() {
+    String catalogName = "barCatalog";
+    Optional<Catalog> defaultCatalog = Catalogs.loadCatalog(conf, catalogName);
+    Assert.assertTrue(defaultCatalog.isPresent());
+    Assertions.assertThat(defaultCatalog.get()).isInstanceOf(HiveCatalog.class);
+    Properties properties = new Properties();
+    properties.put(InputFormatConfig.CATALOG_NAME, catalogName);
+    Assert.assertTrue(Catalogs.hiveCatalog(conf, properties));
+  }
+
+  @Test
+  public void testLoadCatalogHive() {
+    String catalogName = "barCatalog";
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogUtil.ICEBERG_CATALOG_TYPE),
+        CatalogUtil.ICEBERG_CATALOG_TYPE_HIVE);
+    Optional<Catalog> hiveCatalog = Catalogs.loadCatalog(conf, catalogName);
+    Assert.assertTrue(hiveCatalog.isPresent());
+    Assertions.assertThat(hiveCatalog.get()).isInstanceOf(HiveCatalog.class);
+    Properties properties = new Properties();
+    properties.put(InputFormatConfig.CATALOG_NAME, catalogName);
+    Assert.assertTrue(Catalogs.hiveCatalog(conf, properties));
+  }
+
+  @Test
+  public void testLoadCatalogHadoop() {
+    String catalogName = "barCatalog";
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogUtil.ICEBERG_CATALOG_TYPE),
+        CatalogUtil.ICEBERG_CATALOG_TYPE_HADOOP);
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogProperties.WAREHOUSE_LOCATION),
+        "/tmp/mylocation");
+    Optional<Catalog> hadoopCatalog = Catalogs.loadCatalog(conf, catalogName);
+    Assert.assertTrue(hadoopCatalog.isPresent());
+    Assertions.assertThat(hadoopCatalog.get()).isInstanceOf(HadoopCatalog.class);
+    Assert.assertEquals("HadoopCatalog{name=barCatalog, location=/tmp/mylocation}", hadoopCatalog.get().toString());
+    Properties properties = new Properties();
+    properties.put(InputFormatConfig.CATALOG_NAME, catalogName);
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, properties));
+  }
+
+  @Test
+  public void testLoadCatalogHadoopWithLegacyWarehouseLocation() {
+    String catalogName = "barCatalog";
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogUtil.ICEBERG_CATALOG_TYPE),
+        CatalogUtil.ICEBERG_CATALOG_TYPE_HADOOP);
+    conf.set(InputFormatConfig.HADOOP_CATALOG_WAREHOUSE_LOCATION, "/tmp/mylocation");
+    Optional<Catalog> hadoopCatalog = Catalogs.loadCatalog(conf, catalogName);
+    Assert.assertTrue(hadoopCatalog.isPresent());
+    Assertions.assertThat(hadoopCatalog.get()).isInstanceOf(HadoopCatalog.class);
+    Assert.assertEquals("HadoopCatalog{name=barCatalog, location=/tmp/mylocation}", hadoopCatalog.get().toString());
+    Properties properties = new Properties();
+    properties.put(InputFormatConfig.CATALOG_NAME, catalogName);
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, properties));
+  }
+
+  @Test
+  public void testLoadCatalogCustom() {
+    String catalogName = "barCatalog";
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogProperties.CATALOG_IMPL),
+        CustomHadoopCatalog.class.getName());
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogProperties.WAREHOUSE_LOCATION),
+        "/tmp/mylocation");
+    Optional<Catalog> customHadoopCatalog = Catalogs.loadCatalog(conf, catalogName);
+    Assert.assertTrue(customHadoopCatalog.isPresent());
+    Assertions.assertThat(customHadoopCatalog.get()).isInstanceOf(CustomHadoopCatalog.class);
+    Properties properties = new Properties();
+    properties.put(InputFormatConfig.CATALOG_NAME, catalogName);
+    Assert.assertFalse(Catalogs.hiveCatalog(conf, properties));
+  }
+
+  @Test
+  public void testLoadCatalogLocation() {
+    Assert.assertFalse(Catalogs.loadCatalog(conf, Catalogs.ICEBERG_HADOOP_TABLE_NAME).isPresent());
+  }
+
+  @Test
+  public void testLoadCatalogUnknown() {
+    String catalogName = "barCatalog";
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogUtil.ICEBERG_CATALOG_TYPE), "fooType");
+    AssertHelpers.assertThrows(
+        "should complain about catalog not supported", UnsupportedOperationException.class,
+        "Unknown catalog type:", () -> Catalogs.loadCatalog(conf, catalogName));
   }
 
   public static class CustomHadoopCatalog extends HadoopCatalog {
 
-    public static final String WAREHOUSE_LOCATION = "warehouse.location";
+    public CustomHadoopCatalog() {
+
+    }
 
     public CustomHadoopCatalog(Configuration conf, String warehouseLocation) {
       super(conf, warehouseLocation);
     }
 
-    public CustomHadoopCatalog(Configuration conf) {
-      this(conf, conf.get(WAREHOUSE_LOCATION));
-    }
   }
 
-  public static class CustomHadoopCatalogLoader implements CatalogLoader {
-    @Override
-    public Catalog load(Configuration conf) {
-      return new CustomHadoopCatalog(conf);
-    }
+  private void setCustomCatalogProperties(String catalogName, String warehouseLocation) {
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogProperties.WAREHOUSE_LOCATION),
+        warehouseLocation);
+    conf.set(InputFormatConfig.catalogPropertyConfigKey(catalogName, CatalogProperties.CATALOG_IMPL),
+        CustomHadoopCatalog.class.getName());
+    conf.set(InputFormatConfig.CATALOG_NAME, catalogName);
   }
 }
